@@ -9,17 +9,17 @@ from contextlib import contextmanager
 from contextvars import ContextVar
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
-from ._common import Action, get_class_instantiator, is_subclass, parser_context
+from ._common import Action, is_subclass, parser_context
 from ._loaders_dumpers import get_loader_exceptions, load_value
 from ._namespace import Namespace, NSKeyError, split_key, split_key_root
-from ._optionals import FilesCompleterMethod, get_config_read_mode
-from ._type_checking import ArgumentParser
+from ._optionals import _get_config_read_mode
+from ._type_checking import ActionsContainer, ArgumentParser
 from ._util import (
-    NoneType,
     Path,
     argument_error,
     change_to_path_dir,
     default_config_option_help,
+    get_import_path,
     get_typehint_origin,
     import_object,
     indent_text,
@@ -46,7 +46,7 @@ def _is_branch_key(parser, key: str) -> bool:
 
 
 def _find_action_and_subcommand(
-    parser: "ArgumentParser",
+    parser: Union[ArgumentParser, ActionsContainer],
     dest: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
 ) -> Tuple[Optional[ArgparseAction], Optional[str]]:
@@ -64,7 +64,7 @@ def _find_action_and_subcommand(
         actions = [a for a in actions if not isinstance(a, exclude)]
     fallback_action = None
     for action in actions:
-        if action.dest == dest:
+        if action.dest == dest or f"--{dest}" in action.option_strings:
             if isinstance(action, _ActionConfigLoad):
                 fallback_action = action
             else:
@@ -83,7 +83,7 @@ def _find_action_and_subcommand(
 
 
 def _find_action(
-    parser: "ArgumentParser",
+    parser: Union[ArgumentParser, ActionsContainer],
     dest: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
 ) -> Optional[ArgparseAction]:
@@ -91,7 +91,7 @@ def _find_action(
 
 
 def _find_parent_action_and_subcommand(
-    parser: "ArgumentParser",
+    parser: ArgumentParser,
     key: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
 ) -> Tuple[Optional[ArgparseAction], Optional[str]]:
@@ -106,7 +106,7 @@ def _find_parent_action_and_subcommand(
 
 
 def _find_parent_action(
-    parser: "ArgumentParser",
+    parser: ArgumentParser,
     key: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
 ) -> Optional[ArgparseAction]:
@@ -139,13 +139,15 @@ def remove_actions(parser, types):
 
 
 def filter_default_actions(actions):
-    default = (_HelpAction, _ActionHelpClassPath, _ActionPrintConfig)
+    from ._completions import ShtabAction
+
+    default = (_HelpAction, _ActionHelpClassPath, _ActionPrintConfig, ShtabAction)
     if isinstance(actions, list):
         return [a for a in actions if not isinstance(a, default)]
     return {k: a for k, a in actions.items() if not isinstance(a, default)}
 
 
-class ActionConfigFile(Action, FilesCompleterMethod):
+class ActionConfigFile(Action):
     """Action to indicate that an argument is a configuration file or a configuration string."""
 
     def __init__(self, **kwargs):
@@ -174,12 +176,17 @@ class ActionConfigFile(Action, FilesCompleterMethod):
 
     @staticmethod
     def _ensure_single_config_argument(container, action):
-        if is_subclass(action, ActionConfigFile) and any(isinstance(a, ActionConfigFile) for a in container._actions):
-            raise ValueError("A parser is only allowed to have a single ActionConfigFile argument.")
+        if (action == "config" or is_subclass(action, ActionConfigFile)) and any(
+            isinstance(a, ActionConfigFile) for a in container._actions
+        ):
+            raise ValueError("A parser is only allowed to have a single 'config' argument.")
 
     @staticmethod
     def _add_print_config_argument(container, action):
         if isinstance(action, ActionConfigFile) and getattr(container, "_print_config", None) is not None:
+            if "%s" in container._print_config:
+                container._print_config = container._print_config % action.dest
+            assert container._print_config.startswith("--")
             container.add_argument(container._print_config, action=_ActionPrintConfig)
 
     @staticmethod
@@ -187,16 +194,16 @@ class ActionConfigFile(Action, FilesCompleterMethod):
         from ._link_arguments import skip_apply_links
 
         with _ActionSubCommands.not_single_subcommand(), previous_config_context(cfg), skip_apply_links():
-            kwargs = {"env": False, "defaults": False, "_skip_check": True, "_fail_no_subcommand": False}
+            kwargs = {"env": False, "defaults": False, "_skip_validation": True, "_fail_no_subcommand": False}
             try:
-                cfg_path: Optional[Path] = Path(value, mode=get_config_read_mode())
+                cfg_path: Optional[Path] = Path(value, mode=_get_config_read_mode())
             except TypeError as ex_path:
                 try:
                     if isinstance(load_value(value), str):
                         raise ex_path
                     cfg_path = None
                     cfg_file = parser.parse_string(value, **kwargs)
-                except (TypeError,) + get_loader_exceptions() as ex_str:
+                except (TypeError, ValueError) + get_loader_exceptions() as ex_str:  # type: ignore[misc]
                     raise TypeError(f'Parser key "{dest}": {ex_str}') from ex_str
             else:
                 cfg_file = parser.parse_path(value, **kwargs)
@@ -206,6 +213,12 @@ class ActionConfigFile(Action, FilesCompleterMethod):
                 cfg[dest] = []
             cfg[dest].append(cfg_path)
 
+    def completer(self, prefix, **kwargs):
+        from ._completions import get_files_completer
+
+        files_completer = get_files_completer()
+        return sorted(files_completer(prefix, **kwargs))
+
 
 previous_config: ContextVar = ContextVar("previous_config", default=None)
 
@@ -243,7 +256,7 @@ class _ActionPrintConfig(Action):
         )
 
     def __call__(self, parser, namespace, value, option_string=None):
-        kwargs = {"subparser": parser, "key": None, "skip_none": False, "skip_check": False}
+        kwargs = {"subparser": parser, "key": None, "skip_none": False, "skip_validation": False}
         valid_flags = {"": None, "comments": "yaml_comments", "skip_default": "skip_default", "skip_null": "skip_none"}
         if value is not None:
             flags = value[0].split(",")
@@ -326,67 +339,79 @@ class _ActionConfigLoad(Action):
     def check_type(self, value, parser):
         return self._load_config(value, parser)
 
-    def instantiate_classes(self, value):
-        instantiator_fn = get_class_instantiator()
-        return instantiator_fn(self.basetype, **value)
-
 
 class _ActionHelpClassPath(Action):
     sub_add_kwargs: Dict[str, Any] = {}
 
-    def __init__(self, baseclass=None, **kwargs):
-        if baseclass is not None:
-            if get_typehint_origin(baseclass) == Union:
-                baseclasses = [c for c in baseclass.__args__ if c is not NoneType]
-                if len(baseclasses) == 1:
-                    baseclass = baseclasses[0]
-            self._baseclass = baseclass
+    def __init__(self, typehint=None, **kwargs):
+        if typehint is not None:
+            self._typehint = typehint
         else:
-            self._baseclass = kwargs.pop("_baseclass")
+            self._typehint = kwargs.pop("_typehint")
             self.update_init_kwargs(kwargs)
             super().__init__(**kwargs)
 
     def update_init_kwargs(self, kwargs):
-        if get_typehint_origin(self._baseclass) == Union:
-            from ._typehints import ActionTypeHint
+        from ._typehints import (
+            get_optional_arg,
+            get_subclass_names,
+            get_subclass_types,
+            get_unaliased_type,
+            is_protocol,
+        )
+
+        typehint = get_unaliased_type(get_optional_arg(self._typehint))
+        if get_typehint_origin(typehint) is not Union:
+            assert "nargs" not in kwargs
+            kwargs["nargs"] = "?"
+        self._basename = iter_to_set_str(get_subclass_names(self._typehint, callable_return=True))
+        self._baseclasses = get_subclass_types(typehint, callable_return=True)
+        assert self._baseclasses
+
+        self._kind = "subclass of"
+        if any(is_protocol(b) for b in self._baseclasses):
+            self._kind = "subclass or implementer of protocol"
 
-            self._basename = iter_to_set_str(
-                c.__name__ for c in self._baseclass.__args__ if ActionTypeHint.is_subclass_typehint(c)
-            )
-        else:
-            self._basename = self._baseclass.__name__
         kwargs.update(
             {
                 "metavar": "CLASS_PATH_OR_NAME",
                 "default": SUPPRESS,
-                "help": f"Show the help for the given subclass of {self._basename} and exit.",
+                "help": f"Show the help for the given {self._kind} {self._basename} and exit.",
             }
         )
 
     def __call__(self, *args, **kwargs):
         if len(args) == 0:
-            kwargs["_baseclass"] = self._baseclass
+            kwargs["_typehint"] = self._typehint
             return type(self)(**kwargs)
-        dest = re.sub("\\.help$", "", self.dest)
-        return self.print_help(args, self._baseclass, dest)
+        return self.print_help(args)
 
-    def print_help(self, call_args, baseclass, dest):
-        from ._typehints import resolve_class_path_by_name
+    def print_help(self, call_args):
+        from ._typehints import (
+            ActionTypeHint,
+            get_optional_arg,
+            get_unaliased_type,
+            implements_protocol,
+            resolve_class_path_by_name,
+        )
 
         parser, _, value, option_string = call_args
         try:
-            val_class = import_object(resolve_class_path_by_name(baseclass, value))
+            typehint = get_unaliased_type(get_optional_arg(self._typehint))
+            if self.nargs == "?" and value is None:
+                val_class = typehint
+            else:
+                val_class = import_object(resolve_class_path_by_name(typehint, value))
         except Exception as ex:
             raise TypeError(f"{option_string}: {ex}") from ex
-        if get_typehint_origin(self._baseclass) == Union:
-            baseclasses = self._baseclass.__args__
-        else:
-            baseclasses = [baseclass]
-        if not any(is_subclass(val_class, b) for b in baseclasses):
-            raise TypeError(f'{option_string}: Class "{value}" is not a subclass of {self._basename}')
-        dest += ".init_args"
-        subparser = type(parser)()
+        if not any(is_subclass(val_class, b) or implements_protocol(val_class, b) for b in self._baseclasses):
+            raise TypeError(f'{option_string}: Class "{value}" is not a {self._kind} {self._basename}')
+        dest = re.sub("\\.help$", "", self.dest)
+        subparser = type(parser)(description=f"Help for {option_string}={get_import_path(val_class)}")
+        if ActionTypeHint.is_callable_typehint(typehint) and hasattr(typehint, "__args__"):
+            self.sub_add_kwargs["skip"] = {max(0, len(typehint.__args__) - 1)}
         subparser.add_class_arguments(val_class, dest, **self.sub_add_kwargs)
+        subparser._inner_parser = True
         remove_actions(subparser, (_HelpAction, _ActionPrintConfig, _ActionConfigLoad))
         args = self.get_args_after_opt(parser.args)
         if args:
@@ -491,7 +516,7 @@ class ActionParser:
 
     def __init__(
         self,
-        parser: Optional["ArgumentParser"] = None,
+        parser: Optional[ArgumentParser] = None,
     ):
         """Initializer for ActionParser instance.
 
@@ -519,7 +544,7 @@ class ActionParser:
         title = kwargs.pop("title", kwargs.pop("help", None))
         description = kwargs.pop("description", subparser.description)
         if len(kwargs) > 0:
-            raise ValueError(f"ActionParser does not accept the following parameters: {set(kwargs.keys())}")
+            raise ValueError(f"ActionParser does not accept the following parameters: {set(kwargs)}")
         if not (len(args) == 1 and args[0].startswith("--")):
             raise ValueError(f"ActionParser only accepts a single optional key but got {args}")
         prefix = args[0][2:]
@@ -533,7 +558,7 @@ class ActionParser:
         for key, action in filter_default_actions(subparser._option_string_actions).items():
             option_string_actions[add_prefix(key)] = action
 
-        isect = set(option_string_actions.keys()).intersection(set(parser._option_string_actions.keys()))
+        isect = set(option_string_actions).intersection(set(parser._option_string_actions))
         if len(isect) > 0:
             raise ValueError(f"ActionParser conflicting keys: {isect}")
 
@@ -555,6 +580,9 @@ class ActionParser:
         base_action_group._actions = filter_default_actions(base_action_group._actions)
         base_action_group._group_actions = filter_default_actions(base_action_group._group_actions)
         extra_action_groups = subparser._action_groups[2:]
+        for group in extra_action_groups:
+            if group.dest is not None:
+                group.dest = dest + "." + group.dest
 
         parser.add_argument(args[0], action=_ActionConfigLoad)
         parser.required_args.update(required_args)
@@ -577,18 +605,18 @@ parse_kwargs: ContextVar = ContextVar("parse_kwargs", default={})
 @contextmanager
 def parent_parsers_context(key, parser):
     prev = parent_parsers.get()
-    curr = prev + [(key, parser)]
-    t = parent_parsers.set(curr)
+    curr = [] if parser is None else prev + [(key, parser)]
+    token = parent_parsers.set(curr)
     try:
         yield
     finally:
-        parent_parsers.reset(t)
+        parent_parsers.reset(token)
 
 
 class _ActionSubCommands(_SubParsersAction):
     """Extension of argparse._SubParsersAction to modify subcommands functionality."""
 
-    parent_parser: "ArgumentParser"
+    parent_parser: ArgumentParser
     env_prefix: str
 
     def add_parser(self, name, **kwargs):
@@ -644,7 +672,7 @@ class _ActionSubCommands(_SubParsersAction):
         if subcommand in self._name_parser_map:
             subparser = self._name_parser_map[subcommand]
             subnamespace = namespace.get(subcommand).clone() if subcommand in namespace else None
-            kwargs = dict(_skip_check=True, **parse_kwargs.get())
+            kwargs = dict(_skip_validation=True, **parse_kwargs.get())
             namespace[subcommand] = subparser.parse_args(arg_strings, namespace=subnamespace, **kwargs)
 
     @staticmethod
@@ -664,11 +692,11 @@ class _ActionSubCommands(_SubParsersAction):
 
     @staticmethod
     def get_subcommands(
-        parser: "ArgumentParser",
+        parser: ArgumentParser,
         cfg: Namespace,
         prefix: str = "",
         fail_no_subcommand: bool = True,
-    ) -> Tuple[Optional[List[str]], Optional[List["ArgumentParser"]]]:
+    ) -> Tuple[Optional[List[str]], Optional[List[ArgumentParser]]]:
         """Returns subcommand names and corresponding subparsers."""
         if parser._subcommands_action is None:
             return None, None
@@ -677,7 +705,7 @@ class _ActionSubCommands(_SubParsersAction):
         require_single = single_subcommand.get()
 
         # Get subcommand settings keys
-        subcommand_keys = [k for k in action.choices.keys() if isinstance(cfg.get(prefix + k), Namespace)]
+        subcommand_keys = [k for k in action.choices if isinstance(cfg.get(prefix + k), Namespace)]
 
         # Get subcommand
         subcommand = None
@@ -707,7 +735,7 @@ class _ActionSubCommands(_SubParsersAction):
                 # If subcommand is required and no subcommand is provided,
                 # present the user with a friendly error message to remind them of
                 # the available subcommands and to select one.
-                available_subcommands = list(action._name_parser_map.keys())
+                available_subcommands = list(action._name_parser_map)
                 if len(available_subcommands) <= 5:
                     candidate_subcommands_str = "{" + ",".join(available_subcommands) + "}"
                 else:
@@ -720,11 +748,11 @@ class _ActionSubCommands(_SubParsersAction):
 
     @staticmethod
     def get_subcommand(
-        parser: "ArgumentParser",
+        parser: ArgumentParser,
         cfg: Namespace,
         prefix: str = "",
         fail_no_subcommand: bool = True,
-    ) -> Tuple[Optional[str], Optional["ArgumentParser"]]:
+    ) -> Tuple[Optional[str], Optional[ArgumentParser]]:
         """Returns a single subcommand name and corresponding subparser."""
         subcommands, subparsers = _ActionSubCommands.get_subcommands(
             parser,
@@ -736,7 +764,7 @@ class _ActionSubCommands(_SubParsersAction):
 
     @staticmethod
     def handle_subcommands(
-        parser: "ArgumentParser",
+        parser: ArgumentParser,
         cfg: Namespace,
         env: Optional[bool],
         defaults: bool,
@@ -757,9 +785,9 @@ class _ActionSubCommands(_SubParsersAction):
             key = prefix + subcommand
             with parent_parsers_context(key, parser):
                 if env:
-                    subnamespace = subparser.parse_env(defaults=defaults, _skip_check=True)
+                    subnamespace = subparser.parse_env(defaults=defaults, _skip_validation=True)
                 elif defaults:
-                    subnamespace = subparser.get_defaults(skip_check=True)
+                    subnamespace = subparser.get_defaults(skip_validation=True)
 
             # Update all subcommand settings
             if subnamespace is not None:
