@@ -3,7 +3,6 @@ import dataclasses
 import inspect
 import logging
 import os
-import sys
 from contextlib import contextmanager
 from contextvars import ContextVar
 from typing import (  # type: ignore[attr-defined]
@@ -11,6 +10,7 @@ from typing import (  # type: ignore[attr-defined]
     Generic,
     List,
     Optional,
+    Protocol,
     Tuple,
     Type,
     TypeVar,
@@ -20,55 +20,61 @@ from typing import (  # type: ignore[attr-defined]
 
 from ._namespace import Namespace
 from ._optionals import (
+    _set_config_read_mode,
+    _set_docstring_parse_options,
+    capture_typing_extension_shadows,
     get_alias_target,
     get_annotated_base_type,
     import_reconplogger,
     is_alias_type,
     is_annotated,
     reconplogger_support,
+    typing_extensions_import,
 )
-from ._type_checking import ArgumentParser
+from ._type_checking import ActionsContainer, ArgumentParser, docstring_parser
 
 __all__ = [
-    "LoggerProperty",
-    "null_logger",
+    "set_parsing_settings",
 ]
 
 ClassType = TypeVar("ClassType")
 
-if sys.version_info < (3, 8):
-    from typing import Callable
+_UnpackGenericAlias = typing_extensions_import("_UnpackAlias")
 
-    InstantiatorCallable = Callable[..., ClassType]
-else:
-    from typing import Protocol
+unpack_meta_types = set()
+if _UnpackGenericAlias:
+    unpack_meta_types.add(_UnpackGenericAlias)
+capture_typing_extension_shadows(_UnpackGenericAlias, "_UnpackGenericAlias", unpack_meta_types)
 
-    class InstantiatorCallable(Protocol):
-        def __call__(self, class_type: Type[ClassType], *args, **kwargs) -> ClassType:
-            pass  # pragma: no cover
+
+class InstantiatorCallable(Protocol):
+    def __call__(self, class_type: Type[ClassType], *args, **kwargs) -> ClassType:
+        pass  # pragma: no cover
 
 
 InstantiatorsDictType = Dict[Tuple[type, bool], InstantiatorCallable]
 
 
-parent_parser: ContextVar["ArgumentParser"] = ContextVar("parent_parser")
+parent_parser: ContextVar[Optional[ArgumentParser]] = ContextVar("parent_parser", default=None)
 parser_capture: ContextVar[bool] = ContextVar("parser_capture", default=False)
 defaults_cache: ContextVar[Optional[Namespace]] = ContextVar("defaults_cache", default=None)
 lenient_check: ContextVar[Union[bool, str]] = ContextVar("lenient_check", default=False)
 load_value_mode: ContextVar[Optional[str]] = ContextVar("load_value_mode", default=None)
-class_instantiators: ContextVar[Optional[InstantiatorsDictType]] = ContextVar("class_instantiators")
+class_instantiators: ContextVar[Optional[InstantiatorsDictType]] = ContextVar("class_instantiators", default=None)
 nested_links: ContextVar[List[dict]] = ContextVar("nested_links", default=[])
+applied_instantiation_links: ContextVar[Optional[set]] = ContextVar("applied_instantiation_links", default=None)
 
 
-parser_context_vars = dict(
-    parent_parser=parent_parser,
-    parser_capture=parser_capture,
-    defaults_cache=defaults_cache,
-    lenient_check=lenient_check,
-    load_value_mode=load_value_mode,
-    class_instantiators=class_instantiators,
-    nested_links=nested_links,
-)
+parser_context_vars = {
+    "parent_parser": parent_parser,
+    "parser_capture": parser_capture,
+    "defaults_cache": defaults_cache,
+    "lenient_check": lenient_check,
+    "load_value_mode": load_value_mode,
+    "class_instantiators": class_instantiators,
+    "nested_links": nested_links,
+    "applied_instantiation_links": applied_instantiation_links,
+}
 
 
 @contextmanager
@@ -85,6 +91,111 @@ def parser_context(**kwargs):
             context_var.reset(token)
 
 
+parsing_settings = {
+    "validate_defaults": False,
+    "parse_optionals_as_positionals": False,
+}
+
+
+def set_parsing_settings(
+    *,
+    validate_defaults: Optional[bool] = None,
+    config_read_mode_urls_enabled: Optional[bool] = None,
+    config_read_mode_fsspec_enabled: Optional[bool] = None,
+    docstring_parse_style: Optional["docstring_parser.DocstringStyle"] = None,
+    docstring_parse_attribute_docstrings: Optional[bool] = None,
+    parse_optionals_as_positionals: Optional[bool] = None,
+) -> None:
+    """
+    Modify settings that affect the parsing behavior.
+
+    Args:
+        validate_defaults: Whether default values must be valid according to the
+            argument type. The default is False, meaning no default validation,
+            like in argparse.
+        config_read_mode_urls_enabled: Whether to read config files from URLs
+            using requests package. Default is False.
+        config_read_mode_fsspec_enabled: Whether to read config files from
+            fsspec supported file systems. Default is False.
+        docstring_parse_style: The docstring style to expect. Default is
+            DocstringStyle.AUTO.
+        docstring_parse_attribute_docstrings: Whether to parse attribute
+            docstrings (slower). Default is False.
+        parse_optionals_as_positionals: [EXPERIMENTAL] If True, the parser will
+            take extra positional command line arguments as values for optional
+            arguments. This means that optional arguments can be given by name
+            --key=value as usual, but also as positional. The extra positionals
+            are applied to optionals in the order that they were added to the
+            parser. By default, this is False.
+    """
+    # validate_defaults
+    if isinstance(validate_defaults, bool):
+        parsing_settings["validate_defaults"] = validate_defaults
+    elif validate_defaults is not None:
+        raise ValueError(f"validate_defaults must be a boolean, but got {validate_defaults}.")
+    # config_read_mode
+    if config_read_mode_urls_enabled is not None:
+        _set_config_read_mode(urls_enabled=config_read_mode_urls_enabled)
+    if config_read_mode_fsspec_enabled is not None:
+        _set_config_read_mode(fsspec_enabled=config_read_mode_fsspec_enabled)
+    # docstring_parse
+    if docstring_parse_style is not None:
+        _set_docstring_parse_options(style=docstring_parse_style)
+    if docstring_parse_attribute_docstrings is not None:
+        _set_docstring_parse_options(attribute_docstrings=docstring_parse_attribute_docstrings)
+    # parse_optionals_as_positionals
+    if isinstance(parse_optionals_as_positionals, bool):
+        parsing_settings["parse_optionals_as_positionals"] = parse_optionals_as_positionals
+    elif parse_optionals_as_positionals is not None:
+        raise ValueError(f"parse_optionals_as_positionals must be a boolean, but got {parse_optionals_as_positionals}.")
+
+
+def get_parsing_setting(name: str):
+    if name not in parsing_settings:
+        raise ValueError(f"Unknown parsing setting {name}.")
+    return parsing_settings[name]
+
+
+def validate_default(container: ActionsContainer, action: argparse.Action):
+    if action.default is None or not get_parsing_setting("validate_defaults") or not hasattr(action, "_check_type"):
+        return
+    try:
+        with parser_context(parent_parser=container):
+            default = action.default
+            action.default = None
+            action.default = action._check_type_(default)  # type: ignore[attr-defined]
+    except Exception as ex:
+        raise ValueError(f"Default value is not valid: {ex}") from ex
+
+
+def get_optionals_as_positionals_actions(parser, include_positionals=False):
+    from jsonargparse._actions import ActionConfigFile, _ActionConfigLoad, filter_default_actions
+    from jsonargparse._completions import ShtabAction
+    from jsonargparse._typehints import ActionTypeHint
+
+    actions = []
+    for action in filter_default_actions(parser._actions):
+        if isinstance(action, (_ActionConfigLoad, ActionConfigFile, ShtabAction)):
+            continue
+        if ActionTypeHint.is_subclass_typehint(action, all_subtypes=False):
+            continue
+        if action.nargs not in {1, None}:
+            continue
+        if not include_positionals and action.option_strings == []:
+            continue
+        actions.append(action)
+
+    return actions
+
+
+def supports_optionals_as_positionals(parser):
+    return (
+        get_parsing_setting("parse_optionals_as_positionals")
+        and not parser._subcommands_action
+        and not getattr(parser, "_inner_parser", False)
+    )
+
+
 def is_subclass(cls, class_or_tuple) -> bool:
     """Extension of issubclass that supports non-class arguments."""
     try:
@@ -102,6 +213,10 @@ def is_generic_class(cls) -> bool:
     return isinstance(cls, _GenericAlias) and getattr(cls, "__module__", "") != "typing"
 
 
+def is_unpack_typehint(cls) -> bool:
+    return any(isinstance(cls, unpack_type) for unpack_type in unpack_meta_types)
+
+
 def get_generic_origin(cls):
     return cls.__origin__ if is_generic_class(cls) else cls
 
@@ -122,7 +237,7 @@ def get_unaliased_type(cls):
 def is_dataclass_like(cls) -> bool:
     if is_generic_class(cls):
         return is_dataclass_like(cls.__origin__)
-    if not inspect.isclass(cls):
+    if not inspect.isclass(cls) or cls is object:
         return False
     if is_final_class(cls):
         return True
@@ -155,6 +270,12 @@ class ClassInstantiator:
     def __call__(self, class_type: Type[ClassType], *args, **kwargs) -> ClassType:
         for (cls, subclasses), instantiator in self.instantiators.items():
             if class_type is cls or (subclasses and is_subclass(class_type, cls)):
+                param_names = set(inspect.signature(instantiator).parameters)
+                if "applied_instantiation_links" in param_names:
+                    applied_links = applied_instantiation_links.get() or set()
+                    kwargs["applied_instantiation_links"] = {
+                        action.target[0]: action.applied_value for action in applied_links
+                    }
                 return instantiator(class_type, *args, **kwargs)
         return default_class_instantiator(class_type, *args, **kwargs)
 
@@ -195,7 +316,7 @@ def setup_default_logger(data, level, caller):
 def parse_logger(logger: Union[bool, str, dict, logging.Logger], caller):
     if not isinstance(logger, (bool, str, dict, logging.Logger)):
         raise ValueError(f"Expected logger to be an instance of (bool, str, dict, logging.Logger), but got {logger}.")
-    if isinstance(logger, dict) and len(set(logger.keys()) - {"name", "level"}) > 0:
+    if isinstance(logger, dict) and len(set(logger) - {"name", "level"}) > 0:
         value = {k: v for k, v in logger.items() if k not in {"name", "level"}}
         raise ValueError(f"Unexpected data to configure logger: {value}.")
     if logger is False:
@@ -218,7 +339,7 @@ class LoggerProperty:
 
     def __init__(self, *args, logger: Union[bool, str, dict, logging.Logger] = False, **kwargs):
         """Initializer for LoggerProperty class."""
-        self.logger = logger  # type: ignore[assignment]
+        self.logger = logger
         super().__init__(*args, **kwargs)
 
     @property
@@ -240,7 +361,7 @@ class LoggerProperty:
         if logger is None:
             from ._deprecated import deprecation_warning, logger_property_none_message
 
-            deprecation_warning((LoggerProperty.logger, None), logger_property_none_message, stacklevel=2)
+            deprecation_warning((LoggerProperty.logger, None), logger_property_none_message, stacklevel=6)
             logger = False
         if not logger and debug_mode_active():
             logger = {"level": "DEBUG"}
@@ -263,6 +384,6 @@ class Action(LoggerProperty, argparse.Action):
 
     def _check_type_(self, value, **kwargs):
         if not hasattr(self, "_check_type_kwargs"):
-            self._check_type_kwargs = set(inspect.signature(self._check_type).parameters.keys())
+            self._check_type_kwargs = set(inspect.signature(self._check_type).parameters)
         kwargs = {k: v for k, v in kwargs.items() if k in self._check_type_kwargs}
         return self._check_type(value, **kwargs)
