@@ -11,9 +11,9 @@ from pathlib import Path
 from types import ModuleType
 from typing import Any, Callable, Dict, Optional, Set
 
-from ._common import Action
+from ._common import Action, null_logger
+from ._common import LoggerProperty as InternalLoggerProperty
 from ._namespace import Namespace
-from ._optionals import FilesCompleterMethod
 from ._type_checking import ArgumentParser
 
 __all__ = [
@@ -22,7 +22,13 @@ __all__ = [
     "ActionOperators",
     "ActionPath",
     "ActionPathList",
+    "LoggerProperty",
     "ParserError",
+    "get_config_read_mode",
+    "namespace_to_dict",
+    "null_logger",
+    "set_docstring_parse_options",
+    "set_config_read_mode",
     "set_url_support",
     "usage_and_exit_error_handler",
 ]
@@ -119,10 +125,10 @@ def parse_as_dict_patch():
     def patch_parse_method(method_name):
         unpatched_method_name = "_unpatched_" + method_name
 
-        def patched_parse(self, *args, _skip_check: bool = False, **kwargs) -> Union[Namespace, Dict[str, Any]]:
+        def patched_parse(self, *args, _skip_validation: bool = False, **kwargs) -> Union[Namespace, Dict[str, Any]]:
             parse_method = getattr(self, unpatched_method_name)
-            cfg = parse_method(*args, _skip_check=_skip_check, **kwargs)
-            return cfg.as_dict() if self._parse_as_dict and not _skip_check else cfg
+            cfg = parse_method(*args, _skip_validation=_skip_validation, **kwargs)
+            return cfg.as_dict() if self._parse_as_dict and not _skip_validation else cfg
 
         setattr(ArgumentParser, unpatched_method_name, getattr(ArgumentParser, method_name))
         setattr(ArgumentParser, method_name, patched_parse)
@@ -147,7 +153,7 @@ def parse_as_dict_patch():
     # Patch dump
     def patched_dump(self, cfg: Union[Namespace, Dict[str, Any]], *args, **kwargs) -> str:
         if isinstance(cfg, dict):
-            cfg = self.parse_object(cfg, _skip_check=True)
+            cfg = self.parse_object(cfg, _skip_validation=True)
         return self._unpatched_dump(cfg, *args, **kwargs)
 
     ArgumentParser._unpatched_dump = ArgumentParser.dump
@@ -156,7 +162,7 @@ def parse_as_dict_patch():
     # Patch save
     def patched_save(self, cfg: Union[Namespace, Dict[str, Any]], *args, multifile: bool = True, **kwargs) -> None:
         if multifile and isinstance(cfg, dict):
-            cfg = self.parse_object(cfg, _skip_check=True)
+            cfg = self.parse_object(cfg, _skip_validation=True)
         return self._unpatched_save(cfg, *args, multifile=multifile, **kwargs)
 
     ArgumentParser._unpatched_save = ArgumentParser.save
@@ -271,7 +277,7 @@ class ActionPath:
     use as type ``List[<path_type>]`` with ``enable_path=True``.
 """
 )
-class ActionPathList(Action, FilesCompleterMethod):
+class ActionPathList(Action):
     """Action to check and store a list of file paths read from a plain text file or stream."""
 
     def __init__(self, mode: Optional[str] = None, rel: str = "cwd", **kwargs):
@@ -351,16 +357,66 @@ class ActionPathList(Action, FilesCompleterMethod):
     """
     set_url_support was deprecated in v3.12.0 and will be removed in v5.0.0.
     Optional config read modes should now be set using function
-    set_config_read_mode.
+    set_parsing_settings.
 """
 )
 def set_url_support(enabled: bool):
     """Enables/disables URL support for config read mode."""
-    from ._optionals import get_config_read_mode, set_config_read_mode
+    from ._optionals import _get_config_read_mode, _set_config_read_mode
 
-    set_config_read_mode(
+    _set_config_read_mode(
         urls_enabled=enabled,
-        fsspec_enabled=True if "s" in get_config_read_mode() else False,
+        fsspec_enabled=True if "s" in _get_config_read_mode() else False,
+    )
+
+
+@deprecated(
+    """
+    set_config_read_mode was deprecated in v4.39.0 and will be removed in
+    v5.0.0. Optional config read modes should now be set using function
+    set_parsing_settings.
+"""
+)
+def set_config_read_mode(
+    urls_enabled: bool = False,
+    fsspec_enabled: bool = False,
+):
+    """Enables/disables optional config read modes."""
+    from ._optionals import _set_config_read_mode
+
+    _set_config_read_mode(
+        urls_enabled=urls_enabled,
+        fsspec_enabled=fsspec_enabled,
+    )
+
+
+@deprecated(
+    """
+    get_config_read_mode was deprecated in v4.39.0 and will be removed in
+    v5.0.0. The config read mode is internal and thus shouldn't be used.
+"""
+)
+def get_config_read_mode() -> str:
+    """Returns the current config reading mode."""
+    from ._optionals import _get_config_read_mode
+
+    return _get_config_read_mode()
+
+
+@deprecated(
+    """
+    set_docstring_parse_options was deprecated in v4.39.0 and will be removed in
+    v5.0.0. Docstring parse options should now be set using function
+    set_parsing_settings.
+"""
+)
+def set_docstring_parse_options(style=None, attribute_docstrings: Optional[bool] = None):
+    """Sets options for docstring parsing."""
+    from ._optionals import _set_docstring_parse_options
+
+    _set_docstring_parse_options(
+        style=style,
+        attribute_docstrings=attribute_docstrings,
     )
 
 
@@ -370,8 +426,8 @@ cli_return_parser_message = """
 """
 
 
-def deprecation_warning_cli_return_parser():
-    deprecation_warning("CLI.__init__.return_parser", cli_return_parser_message, stacklevel=2)
+def deprecation_warning_cli_return_parser(stacklevel):
+    deprecation_warning("CLI.__init__.return_parser", cli_return_parser_message, stacklevel=stacklevel)
 
 
 logger_property_none_message = """
@@ -468,7 +524,7 @@ class DebugException(Exception):
     this function.
 """
 )
-def usage_and_exit_error_handler(parser: "ArgumentParser", message: str) -> None:
+def usage_and_exit_error_handler(parser: ArgumentParser, message: str) -> None:
     """Prints the usage and exits with error code 2 (same behavior as argparse).
 
     Args:
@@ -497,7 +553,7 @@ class ParserDeprecations:
         self.error_handler = error_handler
 
     @property
-    def error_handler(self) -> Optional[Callable[["ArgumentParser", str], None]]:
+    def error_handler(self) -> Optional[Callable[[ArgumentParser, str], None]]:
         """Property for the error_handler function that is called when there are parsing errors.
 
         :getter: Returns the current error_handler function.
@@ -513,7 +569,7 @@ class ParserDeprecations:
         if error_handler is not False:
             stacklevel = 2
             stack = inspect.stack()[1]
-            if stack.filename.endswith("jsonargparse/_deprecated.py"):
+            if stack.filename.endswith(os.fspath(Path("jsonargparse", "_deprecated.py"))):
                 stacklevel = 5
             deprecation_warning_error_handler(stacklevel)
         if callable(error_handler) or error_handler in {None, False}:
@@ -521,6 +577,43 @@ class ParserDeprecations:
         else:
             raise ValueError("error_handler can be either a Callable or None.")
 
+    @deprecated(
+        """
+        add_dataclass_arguments was deprecated in v4.35.0 and will be removed in
+        v5.0.0. Instead use add_class_arguments.
+    """
+    )
+    def add_dataclass_arguments(self, *args, **kwargs):
+        if "title" in kwargs:
+            kwargs["help"] = kwargs.pop("title")
+        return self.add_class_arguments(*args, **kwargs)
+
+    @deprecated(
+        """
+        ArgumentParser.check_config was deprecated in v4.35.0 and will be removed in
+        v5.0.0. Instead use validate.
+    """
+    )
+    def check_config(self, *args, **kwargs):
+        return self.validate(*args, **kwargs)
+
+
+def deprecated_skip_check(component, kwargs: dict, skip_validation: bool) -> bool:
+    skip_check = kwargs.pop("skip_check", None)
+    if kwargs:
+        raise ValueError(f"Unexpected keyword parameters: {set(kwargs)}")
+    if skip_check is not None:
+        skip_validation = skip_check
+        deprecation_warning(
+            component,
+            (
+                "skip_check parameter was deprecated in v4.35.0 and will be removed in "
+                "v5.0.0. Instead use skip_validation."
+            ),
+            stacklevel=3,
+        )
+    return skip_validation
+
 
 ParserError = ArgumentError
 
@@ -586,3 +679,25 @@ class ActionJsonnetExtVars:
         action = ActionTypeHint(typehint=dict)(**kwargs)
         action.jsonnet_ext_vars = True
         return action
+
+
+@deprecated(
+    """
+    LoggerProperty was deprecated in v4.40.0 and will be removed from the public
+    API in v5.0.0. There is no replacement since jsonargparse is not a logging
+    library. A similar class can be found in reconplogger package.
+"""
+)
+class LoggerProperty(InternalLoggerProperty):
+    """Adds a logger property, intended for internal use."""
+
+
+@deprecated(
+    """
+    namespace_to_dict was deprecated in v4.40.0 and will be removed in v5.0.0.
+    Instead you can use ``.clone().as_dict()`` or ``.as_dict()``.
+"""
+)
+def namespace_to_dict(namespace: Namespace) -> Dict[str, Any]:
+    """Returns a copy of a nested namespace converted into a nested dictionary."""
+    return namespace.clone().as_dict()
