@@ -16,17 +16,18 @@ from ._actions import (
     _ActionPrintConfig,
     _ActionSubCommands,
     _find_parent_action,
+    _find_parent_action_and_subcommand,
     filter_default_actions,
 )
-from ._namespace import Namespace, split_key_leaf
+from ._namespace import Namespace, split_key, split_key_leaf
 from ._parameter_resolvers import get_signature_parameters
-from ._type_checking import ArgumentParser, _ArgumentGroup
+from ._type_checking import ArgumentGroup, ArgumentParser
 
 __all__ = ["ArgumentLinking"]
 
 
 def find_parent_or_child_actions(
-    parser: "ArgumentParser",
+    parser: ArgumentParser,
     key: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
 ) -> Optional[List[ArgparseAction]]:
@@ -44,10 +45,10 @@ def find_parent_or_child_actions(
 
 
 def find_subclass_action_or_class_group(
-    parser: "ArgumentParser",
+    parser: ArgumentParser,
     key: str,
     exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
-) -> Optional[Union[ArgparseAction, "_ArgumentGroup"]]:
+) -> Optional[Union[ArgparseAction, ArgumentGroup]]:
     from ._typehints import ActionTypeHint
 
     action = _find_parent_action(parser, key, exclude=exclude)
@@ -81,7 +82,10 @@ class DirectedGraph:
         for node in [source, target]:
             if node not in self.nodes:
                 self.nodes.append(node)
-        self.edges_dict[self.nodes.index(source)].append(self.nodes.index(target))
+        source_targets_list = self.edges_dict[self.nodes.index(source)]
+        target_index = self.nodes.index(target)
+        if target_index not in source_targets_list:
+            source_targets_list.append(target_index)
 
     def get_topological_order(self):
         exploring = [False] * len(self.nodes)
@@ -97,7 +101,7 @@ class DirectedGraph:
         for target in self.edges_dict[source]:
             if exploring[target]:
                 raise ValueError(
-                    f"Graph has cycles, found while checking {self.nodes[source]} --> " + self.nodes[target]
+                    f"Graph has cycles, found while checking {self.nodes[source]} --> {self.nodes[target]}"
                 )
             elif not visited[target]:
                 self.topological_sort(target, exploring, visited, order)
@@ -139,14 +143,16 @@ class ActionLink(Action):
             ]
 
         # Set and check target action
-        self.target = (target, _find_parent_action(parser, target, exclude=exclude))
+        self.target = (target, find_parent_action_or_group(parser, target, exclude=exclude))
         for key, action in self.source + [self.target]:
             if action is None:
                 raise ValueError(f'No action for key "{key}".')
         assert self.target[1] is not None
 
+        from ._core import ArgumentGroup
         from ._typehints import ActionTypeHint
 
+        is_target_group = isinstance(self.target[1], ArgumentGroup)
         is_target_subclass = ActionTypeHint.is_subclass_typehint(self.target[1], all_subtypes=False, also_lists=True)
         valid_target_init_arg = is_target_subclass and target.startswith(f"{self.target[1].dest}.init_args.")
         valid_target_leaf = self.target[1].dest == target
@@ -154,8 +160,16 @@ class ActionLink(Action):
             prefix = f"{self.target[1].dest}.init_args."
             raise ValueError(f'Target key expected to start with "{prefix}", got "{target}".')
 
+        # Remove target group and child actions
+        if is_target_group:
+            parser._action_groups.remove(self.target[1])
+            del parser.groups[target]
+            for action in list(parser._actions):
+                if action.dest == target or action.dest.startswith(f"{target}."):
+                    parser._actions.remove(action)
         # Replace target action with link action
-        if not is_target_subclass or valid_target_leaf:
+        elif not is_target_subclass or valid_target_leaf:
+            assert isinstance(self.target[1], ArgparseAction)
             for key in self.target[1].option_strings:
                 parser._option_string_actions[key] = self
             parser._actions[parser._actions.index(self.target[1])] = self
@@ -165,7 +179,7 @@ class ActionLink(Action):
                     if is_target_subclass:
                         help_dest = f"{self.target[1].dest}.help"
                         for action in group._group_actions:
-                            if action.dest == help_dest:  # type: ignore[union-attr]
+                            if action.dest == help_dest:
                                 group._group_actions.remove(action)
                                 break
                     if group._group_actions and all(isinstance(a, _ActionConfigLoad) for a in group._group_actions):
@@ -178,7 +192,7 @@ class ActionLink(Action):
         if target in parser.required_args:
             parser.required_args.remove(target)
         if is_target_subclass and not valid_target_leaf:
-            sub_add_kwargs = self.target[1].sub_add_kwargs  # type: ignore[attr-defined]
+            sub_add_kwargs = self.target[1].sub_add_kwargs  # type: ignore[union-attr]
             if "linked_targets" not in sub_add_kwargs:
                 sub_add_kwargs["linked_targets"] = set()
             subtarget = target.split(".init_args.", 1)[1]
@@ -204,12 +218,17 @@ class ActionLink(Action):
         help_str: Optional[str]
         if is_target_subclass and not valid_target_leaf:
             type_attr = None
-            help_str = f"Use --{self.target[1].dest}.help CLASS_PATH for details."
+            help_str = f"Use --{self.target[1].dest}.help for details."
         else:
-            type_attr = getattr(self.target[1], "_typehint", self.target[1].type)
-            help_str = self.target[1].help
+            if is_target_group:
+                type_attr = self.target[1].group_class  # type: ignore[union-attr]
+                help_str = self.target[1].title  # type: ignore[union-attr]
+            else:
+                assert isinstance(self.target[1], ArgparseAction)
+                type_attr = getattr(self.target[1], "_typehint", self.target[1].type)
+                help_str = self.target[1].help
             if help_str == import_module("jsonargparse._formatters").empty_help:
-                help_str = f"Target argument '{self.target[1].dest}' lacks type and help"
+                help_str = f"Target '{self.target[1].dest}' lacks type and help"
 
         super().__init__(
             [link_str],
@@ -268,7 +287,7 @@ class ActionLink(Action):
             raise ValueError(f"Call to compute_fn of link '{link}' with args ({args}) failed: {ex}") from ex
 
     @staticmethod
-    def apply_parsing_links(parser: "ArgumentParser", cfg: Namespace) -> None:
+    def apply_parsing_links(parser: ArgumentParser, cfg: Namespace) -> None:
         if apply_config_skip.get() or _ActionPrintConfig.is_print_config_requested(parser):
             return
 
@@ -360,6 +379,7 @@ class ActionLink(Action):
             else:
                 value = action.call_compute_fn(source_objects)
             ActionLink.set_target_value(action, value, cfg, parser.logger)
+            action.applied_value = value
             applied_links.add(action)
             parser.logger.debug(f"Applied link '{action.option_strings[0]}'.")
 
@@ -388,8 +408,9 @@ class ActionLink(Action):
 
         if ActionTypeHint.is_subclass_typehint(target_action, all_subtypes=False, also_lists=True):
             if target_key == target_action.dest:
-                target_action._check_type(value)  # type: ignore[attr-defined]
+                target_action._check_type(value)  # type: ignore[union-attr]
             else:
+                assert isinstance(target_action.dest, str)
                 parent = cfg.get(target_action.dest)
                 child_key = target_key[len(target_action.dest) + 1 :]
                 if isinstance(parent, list) and any(isinstance(i, Namespace) and child_key in i for i in parent):
@@ -406,11 +427,27 @@ class ActionLink(Action):
     def instantiation_order(parser):
         actions = get_link_actions(parser, "instantiate")
         if actions:
+            targets = set()
             graph = DirectedGraph()
+
+            # Add instantiation links as edges
             for action in actions:
                 target = re.sub(r"\.init_args$", "", split_key_leaf(action.target[0])[0])
                 for _, source_action in action.source:
                     graph.add_edge(source_action.dest, target)
+                targets.add(target)
+
+            # Add instantiation target prefixes as edges
+            targets = sorted(targets, key=lambda x: len(split_key(x)))
+            seen_targets = {targets[0]}
+            for target in targets[1:]:
+                parts = [x.replace("|", ".") for x in target.replace("init_args.", "init_args|").split(".")]
+                for num in range(len(parts) - 1):
+                    target_prefix = ".".join(parts[: num + 1])
+                    if target_prefix in seen_targets:
+                        graph.add_edge(target, target_prefix)
+                seen_targets.add(target)
+
             return graph.get_topological_order()
         return []
 
@@ -434,7 +471,7 @@ class ActionLink(Action):
             if "." not in target_key:
                 return
             parent_key, _ = split_key_leaf(target_key)
-            if "." in target_key and parent_key in cfg and not cfg[parent_key]:
+            if parent_key in cfg and not cfg[parent_key]:
                 del cfg[parent_key]
 
         for action in [a for a in parser._actions if isinstance(a, ActionLink)]:
@@ -453,7 +490,18 @@ class ActionLink(Action):
                     ActionLink.strip_link_target_keys(subparsers[num], cfg[subcommand])
 
 
-def get_link_actions(parser: "ArgumentParser", apply_on: str, skip=set()) -> List[ActionLink]:
+def find_parent_action_or_group(
+    parser: ArgumentParser,
+    key: str,
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Optional[Union[ArgparseAction, ArgumentGroup]]:
+    action_or_group = _find_parent_action_and_subcommand(parser, key, exclude=exclude)[0]
+    if not action_or_group and parser.groups and key in parser.groups:
+        return parser.groups[key]
+    return action_or_group
+
+
+def get_link_actions(parser: ArgumentParser, apply_on: str, skip=set()) -> List[ActionLink]:
     if not hasattr(parser, "_links_group"):
         return []
     return [a for a in parser._links_group._group_actions if a.apply_on == apply_on and a not in skip]
