@@ -11,7 +11,7 @@ from copy import deepcopy
 from functools import partial
 from importlib import import_module
 from types import MethodType
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
 from ._common import (
     LoggerProperty,
@@ -20,14 +20,16 @@ from ._common import (
     is_dataclass_like,
     is_generic_class,
     is_subclass,
+    is_unpack_typehint,
     parse_logger,
 )
 from ._optionals import get_annotated_base_type, is_annotated, is_pydantic_model, parse_docs
 from ._postponed_annotations import evaluate_postponed_annotations
-from ._stubs_resolver import get_stub_types
+from ._stubs_resolver import get_arg_type, get_stub_types, get_stubs_resolver
 from ._util import (
     ClassFromFunctionBase,
     get_import_path,
+    get_typehint_origin,
     iter_to_set_str,
     unique,
 )
@@ -51,6 +53,10 @@ kinds = inspect._ParameterKind
 ast_assign_type: Tuple[Type[ast.AST], ...] = (ast.AnnAssign, ast.Assign)
 param_kwargs_pop_or_get = "**.pop|get():"
 
+ignore_params = {
+    "transformers.BertModel.from_pretrained": {"config_file_name"},
+}
+
 
 class SourceNotAvailable(Exception):
     "Raised when the source code for some component is not available."
@@ -74,6 +80,10 @@ class ConditionalDefault(UnknownDefault):
 
 
 def get_parameter_origins(component, parent) -> Optional[str]:
+    from ._typehints import get_subclass_types, sequence_origin_types
+
+    if get_typehint_origin(component) in sequence_origin_types:
+        component = get_subclass_types(component, also_lists=True)
     if isinstance(component, tuple):
         assert parent is None or len(component) == len(parent)
         return iter_to_set_str(get_parameter_origins(c, parent[n] if parent else None) for n, c in enumerate(component))
@@ -108,7 +118,7 @@ def is_classmethod(parent, component) -> bool:
 
 
 def is_lambda(value: Any) -> bool:
-    return callable(value) and value.__name__ == "<lambda>"
+    return callable(value) and getattr(value, "__name__", "") == "<lambda>"
 
 
 def ast_str(node):
@@ -162,17 +172,7 @@ def ast_is_call_with_value(node, value_dump) -> bool:
 
 
 ast_constant_attr = {ast.Constant: "value"}
-
-if sys.version_info[:2] == (3, 7):
-    ast_constant_attr.update(
-        {
-            ast.NameConstant: "value",
-            ast.Num: "n",
-            ast.Str: "s",
-        }
-    )
-
-ast_constant_types = tuple(ast_constant_attr.keys())
+ast_constant_types = tuple(ast_constant_attr)
 
 
 def ast_is_constant(node):
@@ -263,11 +263,14 @@ def ast_get_call_keyword_names(node):
     return [kw_node.arg for kw_node in node.keywords if kw_node.arg]
 
 
-def remove_given_parameters(node, params):
+def remove_given_parameters(node, params, removed_params: Optional[set] = None):
     given_args = set(ast_get_call_positional_indexes(node))
     given_kwargs = set(ast_get_call_keyword_names(node))
+    input_params = params
     params = [p for n, p in enumerate(params) if n not in given_args]
     params = [p for p in params if p.name not in given_kwargs]
+    if removed_params is not None and len(params) < len(input_params):
+        removed_params.update(p.name for p in input_params if p.name in given_kwargs)
     return params
 
 
@@ -316,6 +319,35 @@ def replace_generic_type_vars(params: ParamList, parent) -> None:
             param.annotation = replace_type_vars(param.annotation)
 
 
+def unpack_typed_dict_kwargs(params: ParamList, kwargs_idx: int) -> int:
+    kwargs = params[kwargs_idx]
+    annotation = kwargs.annotation
+    if is_unpack_typehint(annotation):
+        params.pop(kwargs_idx)
+        annotation_args: tuple = getattr(annotation, "__args__", ())
+        assert len(annotation_args) == 1, "Unpack requires a single type argument"
+        dict_annotations = annotation_args[0].__annotations__
+        new_params = []
+        for nm, annot in dict_annotations.items():
+            new_params.append(
+                ParamData(
+                    name=nm,
+                    annotation=annot,
+                    default=inspect._empty,
+                    kind=inspect._ParameterKind.KEYWORD_ONLY,
+                    doc=None,
+                    component=kwargs.component,
+                    parent=kwargs.parent,
+                    origin=kwargs.origin,
+                )
+            )
+        # insert in-place
+        assert kwargs_idx == len(params), "trailing params should yield a syntax error"
+        params.extend(new_params)
+        return -1
+    return kwargs_idx
+
+
 def add_stub_types(stubs: Optional[Dict[str, Any]], params: ParamList, component) -> None:
     if not stubs:
         return
@@ -345,17 +377,20 @@ def is_param_subclass_instance_default(param: ParamData) -> bool:
     from ._typehints import ActionTypeHint, get_optional_arg, get_subclass_types
 
     annotation = get_optional_arg(param.annotation)
-    class_types = get_subclass_types(annotation)
-    return (class_types and isinstance(param.default, class_types)) or (
-        is_lambda(param.default)
-        and ActionTypeHint.is_callable_typehint(annotation, all_subtypes=False)
-        and annotation.__args__
-        and ActionTypeHint.is_subclass_typehint(annotation.__args__[-1], all_subtypes=False)
+    class_types = get_subclass_types(annotation, callable_return=True)
+    return bool(
+        (class_types and isinstance(param.default, class_types))
+        or (
+            is_lambda(param.default)
+            and ActionTypeHint.is_callable_typehint(annotation)
+            and getattr(annotation, "__args__", None)
+            and ActionTypeHint.is_subclass_typehint(annotation.__args__[-1], all_subtypes=False)
+        )
     )
 
 
 def split_args_and_kwargs(params: ParamList) -> Tuple[ParamList, ParamList]:
-    args = [p for p in params if p.kind is kinds.POSITIONAL_ONLY]
+    args = [p for p in params if p.kind == kinds.POSITIONAL_ONLY]
     kwargs = [p for p in params if p.kind in {kinds.KEYWORD_ONLY, kinds.POSITIONAL_OR_KEYWORD}]
     return args, kwargs
 
@@ -382,7 +417,7 @@ def group_parameters(params_list: List[ParamList]) -> ParamList:
         return params_list[0]
     grouped = []
     non_get_pop_count = 0
-    params_dict = defaultdict(lambda: [])
+    params_dict = defaultdict(list)
     for params in params_list:
         if not (params[0].origin or "").startswith(param_kwargs_pop_or_get):  # type: ignore[union-attr]
             non_get_pop_count += 1
@@ -465,7 +500,10 @@ def get_component_and_parent(
         method_or_property = method_or_property.__name__
     parent = component = None
     if method_or_property:
-        attr = inspect.getattr_static(get_generic_origin(function_or_class), method_or_property)
+        try:
+            attr = inspect.getattr_static(get_generic_origin(function_or_class), method_or_property)
+        except AttributeError as ex:
+            raise AttributeError(f"Attribute '{method_or_property}' not found in {function_or_class}") from ex
         if is_staticmethod(attr):
             component = getattr(function_or_class, method_or_property)
             return component, parent, method_or_property
@@ -669,10 +707,10 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                     node = default_node.body
                     num_positionals = len(param.annotation.__args__) - 1
                 class_type = self.get_call_class_type(node)
-                subclass_types = get_subclass_types(param.annotation)
+                subclass_types = get_subclass_types(param.annotation, callable_return=True)
                 if not (class_type and subclass_types and is_subclass(class_type, subclass_types)):
                     continue
-                subclass_spec: dict = dict(class_path=get_import_path(class_type), init_args=dict())
+                subclass_spec: dict = {"class_path": get_import_path(class_type), "init_args": {}}
                 for kwarg in node.keywords:
                     if kwarg.arg and ast_is_constant(kwarg.value):
                         subclass_spec["init_args"][kwarg.arg] = ast_get_constant_value(kwarg.value)
@@ -737,6 +775,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             return [], []
 
         params_list = []
+        removed_params: Set[str] = set()
         kwargs_value = kwargs_name and values_to_find[kwargs_name]
         kwargs_value_dump = kwargs_value and ast.dump(kwargs_value)
         for node, source in [(v, s) for k, v, s in values_found if k == kwargs_name]:
@@ -760,7 +799,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                     get_param_args = self.get_node_component(node, source)
                     if get_param_args:
                         params = get_signature_parameters(*get_param_args, logger=self.logger)
-                params = remove_given_parameters(node, params)
+                params = remove_given_parameters(node, params, removed_params)
                 if params:
                     self.add_node_origins(params, node)
                     params_list.append(params)
@@ -775,6 +814,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                     self.log_debug(f"unsupported type of assign: {ast_str(node)}")
 
         params = group_parameters(params_list)
+        params = [p for p in params if p.name not in removed_params]
         return split_args_and_kwargs(params)
 
     def get_parameters_attr_use_in_members(self, attr_name) -> ParamList:
@@ -818,6 +858,12 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             matched = group_parameters(matched)
         return matched or None
 
+    def remove_ignore_parameters(self, params: ParamList) -> ParamList:
+        import_path = get_import_path(self.component)
+        if import_path in ignore_params:
+            params = [p for p in params if p.name not in ignore_params[import_path]]
+        return params
+
     def get_parameters(self) -> ParamList:
         if self.component is None:
             return []
@@ -825,35 +871,18 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             self.component, self.parent, self.logger
         )
         self.replace_param_default_subclass_specs(params)
+        if kwargs_idx >= 0:
+            kwargs_idx = unpack_typed_dict_kwargs(params, kwargs_idx)
         if args_idx >= 0 or kwargs_idx >= 0:
             self.doc_params = doc_params
             with mro_context(self.parent):
                 args, kwargs = self.get_parameters_args_and_kwargs()
             params = replace_args_and_kwargs(params, args, kwargs)
         add_stub_types(stubs, params, self.component)
+        params = self.remove_ignore_parameters(params)
         return params
 
 
-def get_parameters_by_assumptions(
-    function_or_class: Union[Callable, Type],
-    method_name: Optional[str] = None,
-    logger: Union[bool, str, dict, logging.Logger] = True,
-) -> ParamList:
-    component, parent, method_name = get_component_and_parent(function_or_class, method_name)
-    params, args_idx, kwargs_idx, _, stubs = get_signature_parameters_and_indexes(component, parent, logger)
-
-    if parent and (args_idx >= 0 or kwargs_idx >= 0):
-        with mro_context(parent):
-            subparams = get_mro_parameters(method_name, get_parameters_by_assumptions, logger)
-        if subparams:
-            args, kwargs = split_args_and_kwargs(subparams)
-            params = replace_args_and_kwargs(params, args, kwargs)
-
-    params = replace_args_and_kwargs(params, [], [])
-    add_stub_types(stubs, params, component)
-    return params
-
-
 def get_field_data_pydantic1_model(field, name, doc_params):
     default = field.default
     if field.required:
@@ -861,11 +890,11 @@ def get_field_data_pydantic1_model(field, name, doc_params):
     elif field.default_factory:
         default = field.default_factory()
 
-    return dict(
-        annotation=field.annotation,
-        default=default,
-        doc=field.field_info.description or doc_params.get(name),
-    )
+    return {
+        "annotation": field.annotation,
+        "default": default,
+        "doc": field.field_info.description or doc_params.get(name),
+    }
 
 
 def get_field_data_pydantic2_dataclass(field, name, doc_params):
@@ -892,11 +921,11 @@ def get_field_data_pydantic2_dataclass(field, name, doc_params):
         field_type = get_annotated_base_type(field.type)
     else:
         field_type = field.type
-    return dict(
-        annotation=field_type,
-        default=default,
-        doc=doc_params.get(name),
-    )
+    return {
+        "annotation": field_type,
+        "default": default,
+        "doc": doc_params.get(name),
+    }
 
 
 def get_field_data_pydantic2_model(field, name, doc_params):
@@ -906,11 +935,11 @@ def get_field_data_pydantic2_model(field, name, doc_params):
     elif field.default_factory:
         default = field.default_factory()
 
-    return dict(
-        annotation=field.rebuild_annotation(),
-        default=default,
-        doc=field.description or doc_params.get(name),
-    )
+    return {
+        "annotation": field.rebuild_annotation(),
+        "default": default,
+        "doc": field.description or doc_params.get(name),
+    }
 
 
 def get_field_data_attrs(field, name, doc_params):
@@ -922,11 +951,11 @@ def get_field_data_attrs(field, name, doc_params):
     elif isinstance(default, attrs.Factory):
         default = default.factory()
 
-    return dict(
-        annotation=field.type,
-        default=default,
-        doc=doc_params.get(name),
-    )
+    return {
+        "annotation": field.type,
+        "default": default,
+        "doc": doc_params.get(name),
+    }
 
 
 def is_init_field_pydantic2_dataclass(field) -> bool:
@@ -951,6 +980,7 @@ def get_parameters_from_pydantic_or_attrs(
 
     if method_or_property or not (pydantic_support or attrs_support):
         return None
+
     function_or_class = get_unaliased_type(function_or_class)
     fields_iterator = get_field_data = None
     if pydantic_support:
@@ -993,6 +1023,76 @@ def get_parameters_from_pydantic_or_attrs(
                 )
             )
     evaluate_postponed_annotations(params, function_or_class, None, logger)
+
+    return params
+
+
+def get_parameters_from_ast(
+    function_or_class: Union[Callable, Type],
+    method_or_property: Optional[str],
+    logger: logging.Logger,
+) -> Optional[ParamList]:
+    visitor = ParametersVisitor(function_or_class, method_or_property, logger=logger)
+    return visitor.get_parameters()
+
+
+def get_parameters_from_stubs(
+    function_or_class: Union[Callable, Type],
+    method_or_property: Optional[str],
+    logger: logging.Logger,
+) -> Optional[ParamList]:
+    component, parent, _ = get_component_and_parent(function_or_class, method_or_property)
+    try:
+        inspect.signature(component)
+        return None
+    except Exception:
+        pass  # only from stubs if getting signature fails
+    params = None
+    resolver = get_stubs_resolver()
+    stub_import = resolver.get_component_imported_info(component, parent)
+    if stub_import:
+        origin = get_parameter_origins(component, parent)
+        aliases = resolver.get_aliases(stub_import)
+        arg_asts = stub_import.info.ast.args.args + stub_import.info.ast.args.kwonlyargs
+        params = []
+        for num, arg_ast in enumerate(arg_asts):
+            if parent and num == 0:
+                continue
+            try:
+                annotation = get_arg_type(arg_ast.annotation, aliases)
+            except Exception:
+                annotation = inspect._empty
+            params.append(
+                ParamData(
+                    name=arg_ast.arg,
+                    annotation=annotation,
+                    default=UnknownDefault("stubs-resolver"),
+                    kind=inspect._ParameterKind.KEYWORD_ONLY,
+                    component=component,
+                    parent=parent,
+                    origin=origin,
+                )
+            )
+    return params
+
+
+def get_parameters_by_assumptions(
+    function_or_class: Union[Callable, Type],
+    method_name: Optional[str],
+    logger: logging.Logger,
+) -> ParamList:
+    component, parent, method_name = get_component_and_parent(function_or_class, method_name)
+    params, args_idx, kwargs_idx, _, stubs = get_signature_parameters_and_indexes(component, parent, logger)
+
+    if parent and (args_idx >= 0 or kwargs_idx >= 0):
+        with mro_context(parent):
+            subparams = get_mro_parameters(method_name, get_parameters_by_assumptions, logger)
+        if subparams:
+            args, kwargs = split_args_and_kwargs(subparams)
+            params = replace_args_and_kwargs(params, args, kwargs)
+
+    params = replace_args_and_kwargs(params, [], [])
+    add_stub_types(stubs, params, component)
     return params
 
 
@@ -1001,7 +1101,7 @@ def get_signature_parameters(
     method_or_property: Optional[str] = None,
     logger: Union[bool, str, dict, logging.Logger] = True,
 ) -> ParamList:
-    """Get parameters by inspecting ASTs or by inheritance assumptions if source not available.
+    """Get parameters by inspecting ASTs, stubs or by inheritance assumptions.
 
     In contrast to inspect.signature, it follows the use of *args and **kwargs
     attempting to find all accepted named parameters.
@@ -1014,22 +1114,26 @@ def get_signature_parameters(
             the parameters for ``__init__``.
         logger: Useful for debugging. Only logs at ``DEBUG`` level.
     """
+    get_component_and_parent(function_or_class, method_or_property)  # verify input
     logger = parse_logger(logger, "get_signature_parameters")
-    try:
-        params = get_parameters_from_pydantic_or_attrs(function_or_class, method_or_property, logger)
+    params = None
+    for get_parameters in [
+        get_parameters_from_pydantic_or_attrs,
+        get_parameters_from_ast,
+        get_parameters_from_stubs,
+        get_parameters_by_assumptions,
+    ]:
+        try:
+            params = get_parameters(function_or_class, method_or_property, logger)
+        except Exception as ex:
+            logger.debug(
+                "%s failed: function_or_class=%s, method_or_property=%s: %s",
+                get_parameters.__name__,
+                function_or_class,
+                method_or_property,
+                ex,
+                exc_info=ex,
+            )
         if params is not None:
-            return params
-        visitor = ParametersVisitor(function_or_class, method_or_property, logger=logger)
-        return visitor.get_parameters()
-    except Exception as ex:
-        cause = "Source not available"
-        exc_info = None
-        if not isinstance(ex, SourceNotAvailable):
-            cause = "Problems with AST resolving"
-            exc_info = ex
-        logger.debug(
-            f"{cause}, falling back to parameters by assumptions: function_or_class={function_or_class} "
-            f"method_or_property={method_or_property}: {ex}",
-            exc_info=exc_info,
-        )
-        return get_parameters_by_assumptions(function_or_class, method_or_property, logger)
+            break
+    return params or []
