@@ -4,7 +4,6 @@ import dataclasses
 import inspect
 import re
 from argparse import SUPPRESS, ArgumentParser
-from contextlib import suppress
 from typing import Any, Callable, List, Optional, Set, Tuple, Type, Union
 
 from ._actions import _ActionConfigLoad
@@ -14,21 +13,21 @@ from ._common import (
     get_generic_origin,
     get_unaliased_type,
     is_dataclass_like,
+    is_final_class,
     is_subclass,
 )
-from ._optionals import get_doc_short_description, is_pydantic_model, pydantic_support
-from ._parameter_resolvers import (
-    ParamData,
-    get_parameter_origins,
-    get_signature_parameters,
-)
+from ._namespace import Namespace
+from ._optionals import attrs_support, get_doc_short_description, is_pydantic_model, pydantic_support
+from ._parameter_resolvers import ParamData, get_parameter_origins, get_signature_parameters
 from ._typehints import (
     ActionTypeHint,
     LazyInitBaseClass,
     callable_instances,
+    get_subclass_names,
     is_optional,
+    not_required_types,
 )
-from ._util import get_import_path, get_private_kwargs, iter_to_set_str
+from ._util import NoneType, get_private_kwargs, get_typehint_origin, iter_to_set_str
 from .typing import register_pydantic_type
 
 __all__ = [
@@ -50,7 +49,7 @@ class SignatureArguments(LoggerProperty):
         nested_key: Optional[str] = None,
         as_group: bool = True,
         as_positional: bool = False,
-        default: Optional[LazyInitBaseClass] = None,
+        default: Optional[Union[dict, Namespace, LazyInitBaseClass, Type]] = None,
         skip: Optional[Set[Union[str, int]]] = None,
         instantiate: bool = True,
         fail_untyped: bool = True,
@@ -66,9 +65,9 @@ class SignatureArguments(LoggerProperty):
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
-            default: Default value used to override parameter defaults. Must be lazy_instance.
+            default: Default value used to override parameter defaults.
             skip: Names of parameters or number of positionals that should be skipped.
-            instantiate: Whether the class group should be instantiated by :code:`instantiate_classes`.
+            instantiate: Whether the class group should be instantiated by ``instantiate_classes``.
             fail_untyped: Whether to raise exception if a required parameter does not have a type.
             sub_configs: Whether subclass type hints should be loadable from inner config file.
 
@@ -79,11 +78,27 @@ class SignatureArguments(LoggerProperty):
             ValueError: When not given a class.
             ValueError: When there are required parameters without at least one valid type.
         """
-        if not inspect.isclass(get_generic_origin(get_unaliased_type(theclass))):
-            raise ValueError(f'Expected "theclass" parameter to be a class type, got: {theclass}.')
-        if default and not (isinstance(default, LazyInitBaseClass) and isinstance(default, theclass)):
-            raise ValueError(f'Expected "default" parameter to be a lazy instance of the class, got: {default}.')
-        linked_targets = get_private_kwargs(kwargs, linked_targets=None)
+        unaliased_class_type = get_unaliased_type(theclass)
+        if not inspect.isclass(get_generic_origin(unaliased_class_type)):
+            raise ValueError(f"Expected 'theclass' parameter to be a class type, got: {theclass}")
+        if not (
+            isinstance(default, (NoneType, dict, Namespace))
+            or (isinstance(default, LazyInitBaseClass) and isinstance(default, unaliased_class_type))
+            or (
+                not is_final_class(default.__class__)
+                and is_dataclass_like(default.__class__)
+                and isinstance(default, unaliased_class_type)
+            )
+        ):
+            raise ValueError(
+                f"Expected 'default' to be dict, Namespace, lazy instance or dataclass-like, got: {default}"
+            )
+        linked_targets, help_, _ = get_private_kwargs(
+            kwargs,
+            linked_targets=None,
+            help=None,
+            required=None,  # Ignored because provided when adding signatures, remove with dataclass inheritance support
+        )
 
         added_args = self._add_signature_arguments(
             theclass,
@@ -96,12 +111,23 @@ class SignatureArguments(LoggerProperty):
             sub_configs=sub_configs,
             instantiate=instantiate,
             linked_targets=linked_targets,
+            help=help_,
         )
 
         if default:
             skip = skip or set()
             prefix = nested_key + "." if nested_key else ""
-            defaults = default.lazy_get_init_args()
+            defaults = default
+            if isinstance(default, LazyInitBaseClass):
+                defaults = default.lazy_get_init_args().as_dict()
+            elif is_dataclass_like(default.__class__):
+                defaults = dataclass_to_dict(default)
+                args = {k[len(prefix) :] for k in added_args}
+                skip_not_added = [k for k in defaults if k not in args]
+                if skip_not_added:
+                    skip.update(skip_not_added)  # skip init=False
+            elif isinstance(default, Namespace):
+                defaults = default.as_dict()
             if defaults:
                 defaults = {prefix + k: v for k, v in defaults.items() if k not in skip}
                 self.set_defaults(**defaults)  # type: ignore[attr-defined]
@@ -218,6 +244,7 @@ class SignatureArguments(LoggerProperty):
         sub_configs: bool = False,
         instantiate: bool = True,
         linked_targets: Optional[Set[str]] = None,
+        help: Optional[str] = None,
     ) -> List[str]:
         """Adds arguments from parameters of objects based on signatures and docstrings.
 
@@ -230,7 +257,7 @@ class SignatureArguments(LoggerProperty):
             skip: Names of parameters or number of positionals that should be skipped.
             fail_untyped: Whether to raise exception if a required parameter does not have a type.
             sub_configs: Whether subclass type hints should be loadable from inner config file.
-            instantiate: Whether the class group should be instantiated by :code:`instantiate_classes`.
+            instantiate: Whether the class group should be instantiated by ``instantiate_classes``.
 
         Returns:
             The list of arguments added.
@@ -261,7 +288,10 @@ class SignatureArguments(LoggerProperty):
                 )
 
         ## Create group if requested ##
-        doc_group = get_doc_short_description(function_or_class, method_name, logger=self.logger)
+        if help is not None:
+            doc_group = help
+        else:
+            doc_group = get_doc_short_description(function_or_class, method_name, logger=self.logger)
         component = getattr(function_or_class, method_name) if method_name else function_or_class
         container = self._create_group_if_requested(
             component,
@@ -309,8 +339,11 @@ class SignatureArguments(LoggerProperty):
         annotation = param.annotation
         if default == inspect_empty:
             default = param.default
-            if default == inspect_empty and is_optional(annotation):
-                default = None
+            if default == inspect_empty:
+                if is_optional(annotation):
+                    default = None
+                elif get_typehint_origin(annotation) in not_required_types:
+                    default = SUPPRESS
         is_required = default == inspect_empty
         src = get_parameter_origins(param.component, param.parent)
         skip_message = f'Skipping parameter "{name}" from "{src}" because of: '
@@ -318,14 +351,12 @@ class SignatureArguments(LoggerProperty):
             annotation = Any
             default = None if is_required else default
             is_required = False
+        is_required_link_target = False
         if is_required and linked_targets is not None and name in linked_targets:
             default = None
             is_required = False
-        if (
-            kind in {kinds.VAR_POSITIONAL, kinds.VAR_KEYWORD}
-            or (not is_required and name[0] == "_")
-            or (annotation == inspect_empty and not is_required and default is None)
-        ):
+            is_required_link_target = True
+        if kind in {kinds.VAR_POSITIONAL, kinds.VAR_KEYWORD} or (not is_required and name[0] == "_"):
             return
         elif skip and name in skip:
             self.logger.debug(skip_message + "Parameter requested to be skipped.")
@@ -338,7 +369,7 @@ class SignatureArguments(LoggerProperty):
             kwargs["help"] = param.doc
         if not is_required:
             kwargs["default"] = default
-            if default is None and not is_optional(annotation, object):
+            if default is None and not is_optional(annotation, object) and not is_required_link_target:
                 annotation = Optional[annotation]
         elif not as_positional:
             kwargs["required"] = True
@@ -399,9 +430,10 @@ class SignatureArguments(LoggerProperty):
                 kwargs.update(sub_add_kwargs)
             with ActionTypeHint.allow_default_instance_context():
                 action = container.add_argument(*args, **kwargs)
-            action.sub_add_kwargs = sub_add_kwargs
-            if is_subclass_typehint and len(subclass_skip) > 0:
-                action.sub_add_kwargs["skip"] = subclass_skip
+            if action is not None:  # None when class without any parameters
+                action.sub_add_kwargs = sub_add_kwargs
+                if is_subclass_typehint and len(subclass_skip) > 0:
+                    action.sub_add_kwargs["skip"] = subclass_skip
             added_args.append(dest)
         elif is_required and fail_untyped:
             raise ValueError(
@@ -409,67 +441,6 @@ class SignatureArguments(LoggerProperty):
                 f" type. Parameter '{name}' from '{src}' does not specify a type."
             )
 
-    def add_dataclass_arguments(
-        self,
-        theclass: Type,
-        nested_key: str,
-        default: Optional[Union[Type, dict]] = None,
-        as_group: bool = True,
-        fail_untyped: bool = True,
-        **kwargs,
-    ) -> List[str]:
-        """Adds arguments from a dataclass based on its field types and docstrings.
-
-        Args:
-            theclass: Class from which to add arguments.
-            nested_key: Key for nested namespace.
-            default: Value for defaults. Must be instance of or kwargs for theclass.
-            as_group: Whether arguments should be added to a new argument group.
-            fail_untyped: Whether to raise exception if a required parameter does not have a type.
-
-        Returns:
-            The list of arguments added.
-
-        Raises:
-            ValueError: When not given a dataclass.
-            ValueError: When default is not instance of or kwargs for theclass.
-        """
-        if not is_dataclass_like(theclass):
-            raise ValueError(f'Expected "theclass" argument to be a dataclass-like, given {theclass}')
-
-        doc_group = get_doc_short_description(theclass, logger=self.logger)
-        for key in ["help", "title"]:
-            if key in kwargs and kwargs[key] is not None:
-                doc_group = kwargs.pop(key)
-        group = self._create_group_if_requested(theclass, nested_key, as_group, doc_group, config_load_type=theclass)
-
-        defaults = {}
-        if default is not None:
-            if isinstance(default, dict):
-                with suppress(TypeError):
-                    default = theclass(**default)
-            if not isinstance(default, get_unaliased_type(theclass)):
-                raise ValueError(
-                    f'Expected "default" argument to be an instance of "{theclass.__name__}" '
-                    f"or its kwargs dict, given {default}"
-                )
-            defaults = dataclass_to_dict(default)
-
-        added_args: List[str] = []
-        param_kwargs = {k: v for k, v in kwargs.items() if k == "sub_configs"}
-        for param in get_signature_parameters(theclass, None, logger=self.logger):
-            self._add_signature_parameter(
-                group,
-                nested_key,
-                param,
-                added_args,
-                fail_untyped=fail_untyped,
-                default=defaults.get(param.name, inspect_empty),
-                **param_kwargs,
-            )
-
-        return added_args
-
     def add_subclass_arguments(
         self,
         baseclass: Union[Type, Tuple[Type, ...]],
@@ -510,8 +481,8 @@ class SignatureArguments(LoggerProperty):
             raise ValueError("Not allowed for dataclass-like classes.")
         if type(baseclass) is not tuple:
             baseclass = (baseclass,)  # type: ignore[assignment]
-        if not baseclass or not all(inspect.isclass(c) for c in baseclass):
-            raise ValueError(f"Expected 'baseclass' argument to be a class or a tuple of classes: {baseclass}")
+        if not baseclass or not all(ActionTypeHint.is_subclass_typehint(c, also_lists=True) for c in baseclass):
+            raise ValueError(f"Expected 'baseclass' to be a subclass type or a tuple of subclass types: {baseclass}")
 
         doc_group = None
         if len(baseclass) == 1:  # type: ignore[arg-type]
@@ -530,7 +501,7 @@ class SignatureArguments(LoggerProperty):
         if skip is not None:
             skip = {f"{nested_key}.init_args." + s for s in skip}
         param = ParamData(name=nested_key, annotation=Union[baseclass], component=baseclass)
-        str_baseclass = iter_to_set_str(get_import_path(x) for x in baseclass)
+        str_baseclass = iter_to_set_str(get_subclass_names(param.annotation))
         kwargs.update(
             {
                 "metavar": metavar,
@@ -538,7 +509,7 @@ class SignatureArguments(LoggerProperty):
             }
         )
         if "default" not in kwargs:
-            kwargs["default"] = SUPPRESS
+            kwargs["default"] = None
         self._add_signature_parameter(
             group, None, param, added_args, skip, sub_configs=True, instantiate=instantiate, **kwargs
         )
@@ -566,17 +537,23 @@ class SignatureArguments(LoggerProperty):
                     doc_group = str(obj[0])
                 else:
                     doc_group = str(obj)
-            name = obj.__name__ if nested_key is None else nested_key
+            name = get_object_name(obj) if nested_key is None else nested_key
             group = self.add_argument_group(strip_title(doc_group), name=name)
             if config_load and nested_key is not None:
                 group.add_argument("--" + nested_key, action=_ActionConfigLoad(basetype=config_load_type))
             if inspect.isclass(obj) and nested_key is not None and instantiate:
-                group.dest = nested_key
+                group.dest = nested_key.replace("-", "_")
                 group.group_class = obj
                 group.instantiate_class = group_instantiate_class
         return group
 
 
+def get_object_name(obj) -> str:
+    if hasattr(obj, "__name__"):
+        return obj.__name__
+    return str(obj).split(".")[-1].replace("[", "_").replace("]", "")
+
+
 def group_instantiate_class(group, cfg):
     try:
         value, parent, key = cfg.get_value_and_parent(group.dest)
@@ -603,8 +580,13 @@ def dataclass_to_dict(value) -> dict:
         pydantic_model = is_pydantic_model(type(value))
         if pydantic_model:
             return value.dict() if pydantic_model == 1 else value.model_dump()
-    if isinstance(value, LazyInitBaseClass):
-        return value.lazy_get_init_data().as_dict()
+
+    if attrs_support:
+        import attrs
+
+        is_attrs_dataclass = attrs.has(type(value))
+        if is_attrs_dataclass:
+            return attrs.asdict(value)
     return dataclasses.asdict(value)
 
 
