@@ -1,14 +1,17 @@
 from typing import TYPE_CHECKING
 
 __all__ = [
-    "_ArgumentGroup",
+    "ArgumentGroup",
+    "ActionsContainer",
     "ArgumentParser",
+    "docstring_parser",
     "ruyamlCommentedMap",
 ]
 
 if TYPE_CHECKING:  # pragma: no cover
+    import docstring_parser
     from ruyaml.comments import CommentedMap as ruyamlCommentedMap
 
-    from ._core import ArgumentParser, _ArgumentGroup
+    from ._core import ActionsContainer, ArgumentGroup, ArgumentParser
 else:
     globals().update({k: None for k in __all__})
