@@ -37,10 +37,10 @@ from ._common import (
     parser_context,
 )
 from ._deprecated import PathDeprecations
-from ._loaders_dumpers import json_dump, load_value
+from ._loaders_dumpers import json_compact_dump, load_value
 from ._optionals import (
+    _get_config_read_mode,
     fsspec_support,
-    get_config_read_mode,
     import_fsspec,
     import_requests,
     url_support,
@@ -135,7 +135,7 @@ def parse_value_or_config(
     cfg_path = None
     if enable_path and type(value) is str and value != "-":
         try:
-            cfg_path = Path(value, mode=get_config_read_mode())
+            cfg_path = Path(value, mode=_get_config_read_mode())
         except TypeError:
             pass
         else:
@@ -216,8 +216,12 @@ def get_import_path(value: Any) -> Optional[str]:
     """Returns the shortest dot import path for the given object."""
     path = None
     value = get_generic_origin(value)
-    module_path = getattr(value, "__module__", None)
-    qualname = getattr(value, "__qualname__", "")
+    if hasattr(value, "__self__") and inspect.isclass(value.__self__) and inspect.ismethod(value):
+        module_path = getattr(value.__self__, "__module__", None)
+        qualname = f"{value.__self__.__name__}.{value.__name__}"
+    else:
+        module_path = getattr(value, "__module__", None)
+        qualname = getattr(value, "__qualname__", "")
 
     if module_path is None:
         path = unresolvable_import_paths.get(value)
@@ -233,7 +237,7 @@ def get_import_path(value: Any) -> Optional[str]:
     if not path:
         raise ValueError(f"Not possible to determine the import path for object {value}.")
 
-    if qualname and module_path and "." in module_path:
+    if qualname and module_path and ("." in qualname or "." in module_path):
         module_parts = module_path.split(".")
         for num in range(len(module_parts)):
             module_path = ".".join(module_parts[: num + 1])
@@ -241,7 +245,10 @@ def get_import_path(value: Any) -> Optional[str]:
             if "." in qualname:
                 obj_name, attr = qualname.rsplit(".", 1)
                 obj = getattr(module, obj_name, None)
-                if getattr(obj, attr, None) is value:
+                if getattr(module, attr, None) is value:
+                    path = module_path + "." + attr
+                    break
+                elif getattr(obj, attr, None) == value:
                     path = module_path + "." + qualname
                     break
             elif getattr(module, qualname, None) is value:
@@ -262,8 +269,12 @@ def object_path_serializer(value):
 
 
 def get_typehint_origin(typehint):
-    if not hasattr(typehint, "__origin__") and get_import_path(typehint.__class__) == "types.UnionType":
-        return Union
+    if not hasattr(typehint, "__origin__"):
+        typehint_class = get_import_path(typehint.__class__)
+        if typehint_class == "types.UnionType":
+            return Union
+        if typehint_class in {"typing._TypedDictMeta", "typing_extensions._TypedDictMeta"}:
+            return dict
     return getattr(typehint, "__origin__", None)
 
 
@@ -304,7 +315,7 @@ def change_to_path_dir(path: Optional["Path"]) -> Iterator[Optional[str]]:
 def hash_item(item):
     try:
         if isinstance(item, (dict, list)):
-            item_hash = hash(json_dump(item))
+            item_hash = hash(json_compact_dump(item))
         else:
             item_hash = hash(item)
     except Exception:
@@ -342,7 +353,7 @@ def indent_text(text: str, first_line: bool = True) -> str:
 def get_private_kwargs(data, **kwargs):
     extracted = [data.pop(name, default) for name, default in kwargs.items()]
     if data:
-        raise ValueError(f"Unexpected keyword parameters: {set(data.keys())}")
+        raise ValueError(f"Unexpected keyword parameters: {set(data)}")
     return extracted[0] if len(extracted) == 1 else extracted
 
 
@@ -447,6 +458,31 @@ def resolve_relative_path(path: str) -> str:
     return "/".join(resolved)
 
 
+def get_argument_group_class(parser):
+    import ast
+
+    from ._core import ActionsContainer, ArgumentGroup
+
+    if parser.__class__.add_argument != ActionsContainer.add_argument:
+        try:
+            add_argument = parser.__class__.add_argument
+            source = inspect.getsource(add_argument)
+            source = "class _ArgumentGroupAutoSubclass(ArgumentGroup):\n" + source
+            class_ast = ast.parse(source)
+            code = compile(class_ast, filename="<ast>", mode="exec")
+            namespace = {**add_argument.__globals__, "ArgumentGroup": ArgumentGroup}
+            exec(code, namespace)
+            group_class = namespace["_ArgumentGroupAutoSubclass"]
+            group_class.__module__ = parser.__class__.__module__
+            add_argument.__globals__[group_class.__name__] = group_class
+            return group_class
+        except Exception as ex:
+            parser.logger.debug(
+                f"Failed to create ArgumentGroup subclass based on {parser.__class__.__name__}: {ex}", exc_info=ex
+            )
+    return ArgumentGroup
+
+
 class PathError(TypeError):
     """Exception raised for errors in the Path class."""
 
@@ -501,13 +537,13 @@ class Path(PathDeprecations):
         is_url = False
         is_fsspec = False
         if isinstance(path, Path):
-            self.std_io = path._std_io
+            self._std_io = path._std_io
             is_url = path.is_url
             is_fsspec = path.is_fsspec
             url_data = path._url_data
             cwd = path.cwd
-            abs_path = path._absolute
-            path = path._relative
+            abs_path = path.absolute
+            path = path.relative
         elif isinstance(path, (str, os.PathLike)):
             if path == "-":
                 self._std_io = True
