@@ -1,38 +1,51 @@
 from __future__ import annotations
 
+import asyncio
+import json
 import os
 import sys
-from contextlib import redirect_stderr, redirect_stdout, suppress
+from contextlib import contextmanager, redirect_stderr, redirect_stdout, suppress
 from dataclasses import asdict, dataclass
+from inspect import getmodule as inspect_getmodule
 from io import StringIO
 from pathlib import Path
-from typing import Optional
+from types import ModuleType
+from typing import Callable, Literal, Optional
 from unittest.mock import patch
 
 import pytest
-import yaml
 
-from jsonargparse import CLI, capture_parser, lazy_instance
+from jsonargparse import CLI, auto_cli, auto_parser, capture_parser, lazy_instance
+from jsonargparse._namespace import Namespace
 from jsonargparse._optionals import docstring_parser_support, ruyaml_support
-from jsonargparse._typehints import Literal
 from jsonargparse.typing import final
-from jsonargparse_tests.conftest import skip_if_docstring_parser_unavailable
+from jsonargparse_tests.conftest import json_or_yaml_dump, json_or_yaml_load, skip_if_docstring_parser_unavailable
 
 
 def get_cli_stdout(*args, **kwargs) -> str:
     out = StringIO()
     with redirect_stdout(out), suppress(SystemExit), patch.dict(os.environ, {"COLUMNS": "150"}):
-        CLI(*args, **kwargs)
+        auto_cli(*args, **kwargs)
     return out.getvalue()
 
 
+def simple_main(a1: int = 0, a2: bool = False):
+    pass
+
+
+def test_auto_parser():
+    parser = auto_parser(simple_main)
+    assert parser.parse_args([]) == Namespace(config=None, a1=0, a2=False)
+
+
 # failure cases
 
 
+@pytest.mark.parametrize("cli_fn", [CLI, auto_cli])
 @pytest.mark.parametrize("components", [0, [], {"x": 0}])
-def test_unexpected_components(components):
+def test_unexpected_components(cli_fn, components):
     with pytest.raises(ValueError):
-        CLI(components)
+        cli_fn(components)
 
 
 class ConflictingSubcommandKey:
@@ -42,7 +55,7 @@ class ConflictingSubcommandKey:
 
 def test_conflicting_subcommand_key():
     with pytest.raises(ValueError) as ctx:
-        CLI(ConflictingSubcommandKey, args=["subcommand", "--x=1"])
+        auto_cli(ConflictingSubcommandKey, args=["subcommand", "--x=1"])
     assert ctx.match("subcommand name can't be the same")
 
 
@@ -54,13 +67,14 @@ def single_function(a1: float):
     return a1
 
 
-def test_single_function_return():
-    assert 1.2 == CLI(single_function, args=["1.2"])
+@pytest.mark.parametrize("cli_fn", [CLI, auto_cli])
+def test_single_function_return(cli_fn):
+    assert 1.2 == cli_fn(single_function, args=["1.2"])
 
 
 def test_single_function_set_defaults():
     def run_cli():
-        CLI(single_function, set_defaults={"a1": 3.4})
+        auto_cli(single_function, set_defaults={"a1": 3.4})
 
     parser = capture_parser(run_cli)
     assert 3.4 == parser.get_defaults().a1
@@ -89,7 +103,7 @@ callable_instance = CallableClass()
 
 
 def test_callable_instance():
-    assert 3 == CLI(callable_instance, as_positional=False, args=["--x=3"])
+    assert 3 == auto_cli(callable_instance, as_positional=False, args=["--x=3"])
 
 
 # multiple functions tests
@@ -105,13 +119,13 @@ def cmd2(a2: str = "X"):
 
 
 def test_multiple_functions_return():
-    assert 5 == CLI([cmd1, cmd2], args=["cmd1", "5"])
-    assert "Y" == CLI([cmd1, cmd2], args=["cmd2", "--a2=Y"])
+    assert 5 == auto_cli([cmd1, cmd2], args=["cmd1", "5"])
+    assert "Y" == auto_cli([cmd1, cmd2], args=["cmd2", "--a2=Y"])
 
 
 def test_multiple_functions_set_defaults():
     def run_cli():
-        CLI([cmd1, cmd2], set_defaults={"cmd2.a2": "Z"})
+        auto_cli([cmd1, cmd2], set_defaults={"cmd2.a2": "Z"})
 
     parser = capture_parser(run_cli)
     assert "Z" == parser.parse_args(["cmd2"]).cmd2.a2
@@ -144,7 +158,6 @@ def conditional_function(fn: "Literal['A', 'B']", *args, **kwargs):
 
 
 @pytest.mark.skipif(condition=sys.version_info < (3, 9), reason="python>=3.9 is required")
-@pytest.mark.skipif(condition=not Literal, reason="Literal is required")
 def test_literal_conditional_function():
     out = get_cli_stdout(conditional_function, args=["--help"])
     assert "Conditional arguments" in out
@@ -167,22 +180,22 @@ class Class1:
 
 
 def test_single_class_return():
-    assert ("0", 2) == CLI(Class1, args=["0", "method1", "2"])
-    assert ("3", 4) == CLI(Class1, args=['--config={"i1": "3", "method1": {"m1": 4}}'])
-    assert ("5", 6) == CLI(Class1, args=["5", "method1", '--config={"m1": 6}'])
+    assert ("0", 2) == auto_cli(Class1, args=["0", "method1", "2"])
+    assert ("3", 4) == auto_cli(Class1, args=['--config={"i1": "3", "method1": {"m1": 4}}'])
+    assert ("5", 6) == auto_cli(Class1, args=["5", "method1", '--config={"m1": 6}'])
 
 
 def test_single_class_missing_required_init():
     err = StringIO()
     with redirect_stderr(err), pytest.raises(SystemExit):
-        CLI(Class1, args=['--config={"method1": {"m1": 2}}'])
+        auto_cli(Class1, args=['--config={"method1": {"m1": 2}}'])
     assert '"i1" is required' in err.getvalue()
 
 
 def test_single_class_invalid_method_parameter():
     err = StringIO()
     with redirect_stderr(err), pytest.raises(SystemExit):
-        CLI(Class1, args=['--config={"i1": "0", "method1": {"m1": "A"}}'])
+        auto_cli(Class1, args=['--config={"i1": "0", "method1": {"m1": "A"}}'])
     assert 'key "m1"' in err.getvalue()
 
 
@@ -215,12 +228,12 @@ def test_single_class_help_docstring_parse_error():
 
 def test_single_class_print_config_after_subcommand():
     out = get_cli_stdout(Class1, args=["0", "method1", "2", "--print_config"])
-    assert "m1: 2" == out.strip()
+    assert {"m1": 2} == json_or_yaml_load(out)
 
 
 def test_single_class_print_config_before_subcommand():
     out = get_cli_stdout(Class1, args=["--print_config", "0", "method1", "2"])
-    cfg = yaml.safe_load(out)
+    cfg = json_or_yaml_load(out)
     assert cfg == {"i1": "0", "method1": {"m1": 2}}
 
 
@@ -234,6 +247,25 @@ def test_method_with_config_parameter():
     assert "p1: 1, config: {}" == out.strip()
 
 
+class WithProperty:
+    def __init__(self, num: int = 1):
+        self._num = num
+
+    @property
+    def prop(self):
+        """Description of property"""
+        return self._num + 1
+
+
+def test_class_property_value():
+    assert 3 == auto_cli(WithProperty, args=["--num=2", "prop"])
+    if docstring_parser_support:
+        help_str = get_cli_stdout(WithProperty, args=["--help"])
+        assert "Description of property" in help_str
+        help_str = get_cli_stdout(WithProperty, args=["prop", "--help"])
+        assert "Description of property" in help_str
+
+
 # function and class tests
 
 
@@ -272,7 +304,7 @@ def cmd3():
     ],
 )
 def test_function_and_class_return(expected, args):
-    assert expected == CLI([cmd1, Cmd2, cmd3], args=args)
+    assert expected == auto_cli([cmd1, Cmd2, cmd3], args=args)
 
 
 def test_function_and_class_main_help():
@@ -306,17 +338,17 @@ def test_function_and_class_subsubcommand_help():
 
 def test_function_and_class_print_config_after_subsubcommand():
     out = get_cli_stdout([cmd1, Cmd2, cmd3], args=["Cmd2", "method2", "--print_config"])
-    assert "m2: 0\n" == out
+    assert {"m2": 0} == json_or_yaml_load(out)
 
 
 def test_function_and_class_print_config_in_between_subcommands():
     out = get_cli_stdout([cmd1, Cmd2, cmd3], args=["Cmd2", "--print_config", "method2"])
-    assert "i1: d\nmethod2:\n  m2: 0\n" == out
+    assert {"i1": "d", "method2": {"m2": 0}} == json_or_yaml_load(out)
 
 
 def test_function_and_class_print_config_before_subcommands():
     out = get_cli_stdout([cmd1, Cmd2, cmd3], args=["--print_config", "Cmd2", "method2"])
-    assert "Cmd2:\n  i1: d\n  method2:\n    m2: 0\n" == out
+    assert {"Cmd2": {"i1": "d", "method2": {"m2": 0}}} == json_or_yaml_load(out)
 
 
 @skip_if_docstring_parser_unavailable
@@ -340,40 +372,63 @@ def test_function_and_class_function_without_parameters():
 # automatic components tests
 
 
-def test_automatic_components_empty_context():
+@contextmanager
+def mock_getmodule_locals(parent_fn, locals_list=[]):
+    module_name = "_" + parent_fn.__name__
+
+    mock_module = ModuleType(module_name)
+    for obj in locals_list + [CLI, auto_cli]:
+        setattr(mock_module, obj.__name__, obj)
+    sys.modules[module_name] = mock_module
+
+    for obj in locals_list:
+        obj.__module__ = module_name
+
+    def patched_getmodule(obj, *args):
+        if obj in locals_list or (parent_fn.__name__ in str(obj)):
+            return mock_module
+        return inspect_getmodule(obj, *args)
+
+    with patch("inspect.getmodule", side_effect=patched_getmodule):
+        yield
+        del sys.modules[module_name]
+
+
+@pytest.mark.parametrize("cli_fn", [CLI, auto_cli])
+def test_automatic_components_empty_context(cli_fn):
     def empty_context():
-        CLI()
+        cli_fn()
 
-    with patch("inspect.getmodule") as mock_getmodule:
-        mock_getmodule.return_value = sys.modules["jsonargparse._core"]
-        pytest.raises(ValueError, empty_context)
+    with mock_getmodule_locals(empty_context):
+        with pytest.raises(ValueError, match="Either components parameter must be given or"):
+            empty_context()
 
 
-def test_automatic_components_context_function():
+@pytest.mark.parametrize("cli_fn", [CLI, auto_cli])
+def test_automatic_components_context_function(cli_fn):
+    def function(a1: float):
+        return a1
+
     def non_empty_context_function():
-        def function(a1: float):
-            return a1
+        return cli_fn(args=["6.7"])
 
-        return CLI(args=["6.7"])
-
-    with patch("inspect.getmodule") as mock_getmodule:
-        mock_getmodule.return_value = sys.modules["jsonargparse._core"]
+    with mock_getmodule_locals(non_empty_context_function, [function]):
         assert 6.7 == non_empty_context_function()
 
 
-def test_automatic_components_context_class():
+@pytest.mark.parametrize("cli_fn", [CLI, auto_cli])
+def test_automatic_components_context_class(cli_fn):
+    class ClassX:
+        def __init__(self, i1: str):
+            self.i1 = i1
+
+        def method(self, m1: int):
+            return self.i1, m1
+
     def non_empty_context_class():
-        class ClassX:
-            def __init__(self, i1: str):
-                self.i1 = i1
+        return cli_fn(args=["a", "method", "2"])
 
-            def method(self, m1: int):
-                return self.i1, m1
-
-        return CLI(args=["a", "method", "2"])
-
-    with patch("inspect.getmodule") as mock_getmodule:
-        mock_getmodule.return_value = sys.modules["jsonargparse._core"]
+    with mock_getmodule_locals(non_empty_context_class, [ClassX]):
         assert ("a", 2) == non_empty_context_class()
 
 
@@ -387,13 +442,13 @@ class SettingsClass:
 
 
 def test_dataclass_without_methods_response():
-    settings = CLI(SettingsClass, args=["--p1=x", "--p2=0"], as_positional=False)
+    settings = auto_cli(SettingsClass, args=["--p1=x", "--p2=0"], as_positional=False)
     assert isinstance(settings, SettingsClass)
     assert asdict(settings) == {"p1": "x", "p2": 0}
 
 
 def test_dataclass_without_methods_parser_groups():
-    parser = capture_parser(lambda: CLI(SettingsClass, args=[], as_positional=False))
+    parser = capture_parser(lambda: auto_cli(SettingsClass, args=[], as_positional=False))
     assert parser.groups == {}
 
 
@@ -402,8 +457,8 @@ def test_dataclass_without_methods_parser_groups():
 
 def test_named_components_shallow():
     components = {"cmd1": single_function, "cmd2": callable_instance}
-    assert 3.4 == CLI(components, args=["cmd1", "3.4"])
-    assert 5 == CLI(components, as_positional=False, args=["cmd2", "--x=5"])
+    assert 3.4 == auto_cli(components, args=["cmd1", "3.4"])
+    assert 5 == auto_cli(components, as_positional=False, args=["cmd2", "--x=5"])
     out = get_cli_stdout(components, args=["--help"])
     if docstring_parser_support:
         assert "Description of single_function" in out
@@ -468,9 +523,9 @@ def test_named_components_deep():
     if docstring_parser_support:
         assert "Description of method1" in out
 
-    assert 5.6 == CLI(components, args=["lv1_a", "lv2_x", "--a1=5.6"], **kw)
-    assert 7 == CLI(components, args=["lv1_a", "lv2_y", "lv3_p", "--x=7"], **kw)
-    assert ("w", 9) == CLI(components, args=["lv1_b", "lv2_z", "lv3_q", "--i1=w", "method1", "--m1=9"], **kw)
+    assert 5.6 == auto_cli(components, args=["lv1_a", "lv2_x", "--a1=5.6"], **kw)
+    assert 7 == auto_cli(components, args=["lv1_a", "lv2_y", "lv3_p", "--x=7"], **kw)
+    assert ("w", 9) == auto_cli(components, args=["lv1_b", "lv2_z", "lv3_q", "--i1=w", "method1", "--m1=9"], **kw)
 
 
 # config file tests
@@ -500,9 +555,9 @@ class C:
 
 
 def test_subclass_type_config_file(tmp_cwd):
-    a_yaml = {"class_path": f"{__name__}.A", "init_args": {"p1": "a yaml"}}
-    Path("config.yaml").write_text("a: a.yaml\n")
-    Path("a.yaml").write_text(yaml.safe_dump(a_yaml))
+    a_conf = {"class_path": f"{__name__}.A", "init_args": {"p1": "a yaml"}}
+    Path("config.yaml").write_text(json_or_yaml_dump({"a": "a.yaml"}))
+    Path("a.yaml").write_text(json_or_yaml_dump(a_conf))
 
     out = get_cli_stdout(C, args=["--config=config.yaml", "cmd_a"])
     assert "a yaml\n" == out
@@ -510,8 +565,9 @@ def test_subclass_type_config_file(tmp_cwd):
     out = get_cli_stdout(C, args=["cmd_a", "--help"])
     assert "--config" not in out
 
-    Path("config.yaml").write_text("a: a.yaml\nb: b.yaml\n")
-    Path("b.yaml").write_text(f"class_path: {__name__}.B\ninit_args:\n  a: a.yaml\n")
+    b_conf = {"class_path": f"{__name__}.B", "init_args": {"a": "a.yaml"}}
+    Path("config.yaml").write_text(json_or_yaml_dump({"a": "a.yaml", "b": "b.yaml"}))
+    Path("b.yaml").write_text(json_or_yaml_dump(b_conf))
 
     out = get_cli_stdout(C, args=["--config=config.yaml", "cmd_b"])
     assert "a yaml\n" == out
@@ -528,10 +584,51 @@ def run_bf(b: BF):
 
 
 def test_final_and_subclass_type_config_file(tmp_cwd):
-    a_yaml = {"class_path": f"{__name__}.A", "init_args": {"p1": "a yaml"}}
-    Path("config.yaml").write_text("b: b.yaml\n")
-    Path("b.yaml").write_text("a: a.yaml\n")
-    Path("a.yaml").write_text(yaml.safe_dump(a_yaml))
+    a_conf = {"class_path": f"{__name__}.A", "init_args": {"p1": "a yaml"}}
+    Path("config.yaml").write_text(json_or_yaml_dump({"b": "b.yaml"}))
+    Path("b.yaml").write_text(json_or_yaml_dump({"a": "a.yaml"}))
+    Path("a.yaml").write_text(json_or_yaml_dump(a_conf))
 
-    out = CLI(run_bf, args=["--config=config.yaml"])
+    out = auto_cli(run_bf, args=["--config=config.yaml"])
     assert "a yaml" == out
+
+
+# async tests
+
+
+async def run_async(time: float = 0.1):
+    await asyncio.sleep(time)
+    return "done"
+
+
+def test_async_function():
+    assert "done" == auto_cli(run_async, args=["--time=0.0"])
+
+
+class AsyncMethod:
+    def __init__(self, time: float = 0.1, require_async: bool = False):
+        self.time = time
+        if require_async:
+            self.loop = asyncio.get_event_loop()
+
+    async def run(self):
+        await asyncio.sleep(self.time)
+        return "done"
+
+
+def test_async_method():
+    assert "done" == auto_cli(AsyncMethod, args=["--time=0.0", "run"])
+
+
+async def run_async_instance(cls: Callable[[], AsyncMethod]):
+    return await cls().run()
+
+
+def test_async_instance():
+    config = {
+        "cls": {
+            "class_path": f"{__name__}.AsyncMethod",
+            "init_args": {"time": 0.0, "require_async": True},
+        }
+    }
+    assert "done" == auto_cli(run_async_instance, args=[f"--config={json.dumps(config)}"])
