@@ -1,31 +1,33 @@
 from __future__ import annotations
 
 import dataclasses
-from typing import Any, Dict, Generic, List, Optional, Tuple, TypeVar, Union
+import json
+import sys
+from typing import Any, Dict, Generic, List, Literal, Optional, Tuple, TypeVar, Union
 from unittest.mock import patch
 
 import pytest
-import yaml
 
 from jsonargparse import (
-    ActionConfigFile,
     ArgumentError,
     ArgumentParser,
     Namespace,
     compose_dataclasses,
     lazy_instance,
+    set_parsing_settings,
 )
+from jsonargparse._namespace import NSKeyError
 from jsonargparse._optionals import (
     attrs_support,
     docstring_parser_support,
     pydantic_support,
     pydantic_supports_field_init,
-    set_docstring_parse_options,
     typing_extensions_import,
 )
 from jsonargparse.typing import PositiveFloat, PositiveInt, final
 from jsonargparse_tests.conftest import (
     get_parser_help,
+    json_or_yaml_load,
     skip_if_docstring_parser_unavailable,
 )
 
@@ -44,7 +46,7 @@ class DataClassA:
         a2: a2 help
     """
 
-    a1: PositiveInt = PositiveInt(1)  # type: ignore[valid-type]
+    a1: PositiveInt = PositiveInt(1)
     a2: str = "2"
 
 
@@ -57,23 +59,23 @@ class DataClassB:
         b2: b2 help
     """
 
-    b1: PositiveFloat = PositiveFloat(3.0)  # type: ignore[valid-type]
-    b2: DataClassA = DataClassA()
+    b1: PositiveFloat = PositiveFloat(3.0)
+    b2: DataClassA = DataClassA(a2="x")
 
 
 class MixedClass(int, DataClassA):
     """MixedClass description"""
 
 
-def test_add_dataclass_arguments(parser, subtests):
-    parser.add_dataclass_arguments(DataClassA, "a", default=DataClassA(), title="CustomA title")
-    parser.add_dataclass_arguments(DataClassB, "b", default=DataClassB())
+def test_add_class_arguments(parser, subtests):
+    parser.add_class_arguments(DataClassA, "a", default=DataClassA(), help="CustomA title")
+    parser.add_class_arguments(DataClassB, "b", default=DataClassB())
 
     with subtests.test("get_defaults"):
         cfg = parser.get_defaults()
         assert dataclasses.asdict(DataClassA()) == cfg["a"].as_dict()
         assert dataclasses.asdict(DataClassB()) == cfg["b"].as_dict()
-        dump = yaml.safe_load(parser.dump(cfg))
+        dump = json_or_yaml_load(parser.dump(cfg))
         assert dataclasses.asdict(DataClassA()) == dump["a"]
         assert dataclasses.asdict(DataClassB()) == dump["b"]
 
@@ -96,11 +98,11 @@ def test_add_dataclass_arguments(parser, subtests):
 
     with subtests.test("add failures"):
         with pytest.raises(ValueError):
-            parser.add_dataclass_arguments(1, "c")
+            parser.add_class_arguments(1, "c")
+        with pytest.raises(NSKeyError):
+            parser.add_class_arguments(DataClassB, "c", default=DataClassB(b2=DataClassB()))
         with pytest.raises(ValueError):
-            parser.add_dataclass_arguments(DataClassB, "c", default=DataClassB(b2=DataClassB()))
-        with pytest.raises(ValueError):
-            parser.add_dataclass_arguments(MixedClass, "c")
+            parser.add_class_arguments(MixedClass, "c")
 
 
 @dataclasses.dataclass
@@ -114,8 +116,18 @@ class NestedDefaultsB:
     a: List[NestedDefaultsA]
 
 
+@dataclasses.dataclass
+class NestedDefaultsC:
+    field_with_dash: int = 5
+
+
+@dataclasses.dataclass
+class NestedDefaultsD:
+    c_with_dash: NestedDefaultsC = dataclasses.field(default_factory=NestedDefaultsC)
+
+
 def test_add_dataclass_nested_defaults(parser):
-    parser.add_dataclass_arguments(NestedDefaultsB, "data")
+    parser.add_class_arguments(NestedDefaultsB, "data")
     cfg = parser.parse_args(["--data.a=[{}]"])
     assert cfg.data == Namespace(a=[Namespace(x=[], v=1)])
 
@@ -136,7 +148,7 @@ def test_add_class_with_dataclass_attributes(parser):
     cfg = parser.get_defaults()
     assert dataclasses.asdict(DataClassA()) == cfg.g.a1.as_dict()
     assert dataclasses.asdict(DataClassB()) == cfg.g.a2.as_dict()
-    dump = yaml.safe_load(parser.dump(cfg))
+    dump = json_or_yaml_load(parser.dump(cfg))
     assert dataclasses.asdict(DataClassA()) == dump["g"]["a1"]
     assert dataclasses.asdict(DataClassB()) == dump["g"]["a2"]
 
@@ -187,15 +199,53 @@ def test_add_class_optional_without_default(parser):
     assert parser.parse_args(["--param=null"]) == Namespace(param=None)
 
 
+@dataclasses.dataclass
+class ListOptionalA:
+    x: int
+
+
+@dataclasses.dataclass
+class ListOptionalB:
+    a: Optional[ListOptionalA] = None
+
+
+def test_list_nested_optional_dataclass(parser):
+    parser.add_argument("--b", type=List[ListOptionalB])
+    cfg = parser.parse_args(['--b=[{"a":{"x":1}}]'])
+    assert cfg.b == [Namespace(a=Namespace(x=1))]
+
+
+@dataclasses.dataclass
+class ItemData:
+    x: int = 1
+    y: str = "one"
+
+
+def test_list_append_defaults(parser):
+    parser.add_argument("--list", type=List[ItemData])
+    cfg = parser.parse_args(["--list+={}", '--list+={"x":2,"y":"two"}', '--list+={"x":3}'])
+    assert cfg.list == [Namespace(x=1, y="one"), Namespace(x=2, y="two"), Namespace(x=3, y="one")]
+
+
 def test_add_argument_dataclass_type(parser):
     parser.add_argument("--b", type=DataClassB, default=DataClassB(b1=7.0))
     cfg = parser.get_defaults()
-    assert {"b1": 7.0, "b2": {"a1": 1, "a2": "2"}} == cfg.b.as_dict()
+    assert {"b1": 7.0, "b2": {"a1": 1, "a2": "x"}} == cfg.b.as_dict()
     init = parser.instantiate_classes(cfg)
     assert isinstance(init.b, DataClassB)
     assert isinstance(init.b.b2, DataClassA)
 
 
+def test_add_argument_dataclass_unexpected_keys(parser):
+    parser.add_argument("--b", type=DataClassB)
+    invalid = {
+        "class_path": f"{__name__}.DataClassB",
+    }
+    with pytest.raises(ArgumentError) as ctx:
+        parser.parse_args([f"--b={json.dumps(invalid)}"])
+    ctx.match("Group 'b' does not accept nested key 'class_path'")
+
+
 @dataclasses.dataclass
 class DataRequiredAttr:
     a1: str
@@ -216,11 +266,34 @@ class DataInitFalse:
 
 
 def test_dataclass_field_init_false(parser):
-    added = parser.add_dataclass_arguments(DataInitFalse, "d")
+    added = parser.add_class_arguments(DataInitFalse, "data")
     assert added == []
     assert parser.get_defaults() == Namespace()
-    cfg = parser.parse_args(["--d", "{}"])
-    assert cfg.d == Namespace()
+    cfg = parser.parse_args([])
+    assert cfg == Namespace()
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.data, DataInitFalse)
+
+
+@dataclasses.dataclass
+class NestedDataInitFalse:
+    x: bool = dataclasses.field(default=False, init=False)
+
+
+@dataclasses.dataclass
+class ParentDataInitFalse:
+    y: NestedDataInitFalse = dataclasses.field(default_factory=NestedDataInitFalse)
+
+
+def test_nested_dataclass_field_init_false(parser):
+    parser.add_class_arguments(ParentDataInitFalse, "data")
+    assert parser.get_defaults() == Namespace()
+    cfg = parser.parse_args([])
+    assert cfg == Namespace()
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.data, ParentDataInitFalse)
+    assert isinstance(init.data.y, NestedDataInitFalse)
+    assert init.data.y.x is False
 
 
 @dataclasses.dataclass
@@ -298,6 +371,25 @@ def test_instantiate_dataclass_within_classes(parser):
     assert isinstance(init["class"].data, NestedData)
 
 
+@dataclasses.dataclass
+class RequiredAttr:
+    an_int: int
+
+
+@dataclasses.dataclass
+class NestedRequiredAttr:
+    b: RequiredAttr
+
+
+def test_list_nested_dataclass_required_attr(parser):
+    parser.add_argument("--a", type=List[NestedRequiredAttr])
+    cfg = parser.parse_args(['--a=[{"b": {"an_int": 3}}]'])
+    assert cfg == Namespace(a=[Namespace(b=Namespace(an_int=3))])
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.a[0].b, RequiredAttr)
+    assert isinstance(init.a[0], NestedRequiredAttr)
+
+
 @dataclasses.dataclass
 class WithAttrDocs:
     attr_str: str = "a"
@@ -309,7 +401,7 @@ class WithAttrDocs:
 @skip_if_docstring_parser_unavailable
 @patch.dict("jsonargparse._optionals._docstring_parse_options")
 def test_attribute_docstrings(parser):
-    set_docstring_parse_options(attribute_docstrings=True)
+    set_parsing_settings(docstring_parse_attribute_docstrings=True)
     parser.add_class_arguments(WithAttrDocs)
     help_str = get_parser_help(parser)
     assert "attr_str description (type: str, default: a)" in help_str
@@ -351,7 +443,7 @@ def test_optional_dataclass_type_instantiate():
 
 def test_optional_dataclass_type_dump():
     cfg = parser_optional_data.parse_args(['--data={"p1": "z"}'])
-    assert parser_optional_data.dump(cfg) == "data:\n  p1: z\n  p2: 0\n"
+    assert json_or_yaml_load(parser_optional_data.dump(cfg)) == {"data": {"p1": "z", "p2": 0}}
 
 
 def test_optional_dataclass_type_missing_required_field():
@@ -365,6 +457,28 @@ def test_optional_dataclass_type_null_value():
     assert cfg == parser_optional_data.instantiate_classes(cfg)
 
 
+@dataclasses.dataclass
+class DataWithOptionalB:
+    c: int = 3
+
+
+@dataclasses.dataclass
+class DataWithOptionalA:
+    b: Optional[DataWithOptionalB] = dataclasses.field(default_factory=DataWithOptionalB)
+
+
+def data_with_optional(a: DataWithOptionalA):
+    pass
+
+
+def test_dataclass_with_optional_default(parser):
+    parser.add_function_arguments(data_with_optional, "data")
+    cfg = parser.parse_args([])
+    assert cfg.data == Namespace(a=Namespace(b={"c": 3}))
+    init = parser.instantiate_classes(cfg)
+    assert init.data.a == DataWithOptionalA()
+
+
 @dataclasses.dataclass
 class SingleParamChange:
     p1: int = 0
@@ -372,10 +486,10 @@ class SingleParamChange:
 
 
 def test_optional_dataclass_single_param_change(parser):
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
     parser.add_argument("--data", type=Optional[SingleParamChange])
     config = {"data": {"p1": 1}}
-    cfg = parser.parse_args([f"--config={config}", "--data.p2=2"])
+    cfg = parser.parse_args([f"--config={json.dumps(config)}", "--data.p2=2"])
     assert cfg.data == Namespace(p1=1, p2=2)
 
 
@@ -387,7 +501,9 @@ class ModelConfig:
 def test_dataclass_optional_dict_attribute(parser):
     parser.add_argument("--model", type=Optional[ModelConfig], default=ModelConfig(data={"A": 1, "B": 2}))
     cfg = parser.parse_args(["--model.data.A=4"])
-    assert cfg.model.data == {"A": 4, "B": 2}
+    assert cfg.model["data"] == {"A": 4, "B": 2}
+    init = parser.instantiate_classes(cfg)
+    assert init.model == ModelConfig(data={"A": 4, "B": 2})
 
 
 def test_dataclass_in_union_type(parser):
@@ -432,7 +548,7 @@ class SpecificData:
 
 
 def test_nested_generic_dataclass(parser):
-    parser.add_dataclass_arguments(SpecificData, "x")
+    parser.add_class_arguments(SpecificData, "x")
     help_str = get_parser_help(parser).lower()
     assert "--x.y.g1 g1          (required, type: float)" in help_str
     assert "--x.y.g2 [item,...]  (required, type: tuple[float, float])" in help_str
@@ -440,6 +556,71 @@ def test_nested_generic_dataclass(parser):
     assert "--x.y.g4 g4          (required, type: dict[str, union[float, bool]])" in help_str
 
 
+if sys.version_info >= (3, 9):
+    V = TypeVar("V")
+
+    @dataclasses.dataclass(frozen=True)
+    class GenericChild(Generic[V]):
+        value: V
+
+    @dataclasses.dataclass(frozen=True)
+    class GenericBase(Generic[V]):
+        children: tuple[GenericChild[V], ...]
+
+    @dataclasses.dataclass(frozen=True)
+    class GenericSubclass(GenericBase[str]):
+        children: tuple[GenericChild[str], ...]
+
+    def test_generic_dataclass_subclass(parser):
+        parser.add_class_arguments(GenericSubclass, "x")
+        cfg = parser.parse_args(['--x.children=[{"value": "a"}, {"value": "b"}]'])
+        init = parser.instantiate_classes(cfg)
+        assert cfg.x.children == (Namespace(value="a"), Namespace(value="b"))
+        assert isinstance(init.x, GenericSubclass)
+        assert isinstance(init.x.children[0], GenericChild)
+        assert isinstance(init.x.children[1], GenericChild)
+
+
+# union mixture tests
+
+
+@dataclasses.dataclass
+class UnionData:
+    data_a: int = 1
+    data_b: Optional[str] = None
+
+
+class UnionClass:
+    def __init__(self, prm_1: float, prm_2: bool):
+        self.prm_1 = prm_1
+        self.prm_2 = prm_2
+
+
+@pytest.mark.parametrize(
+    "union_type",
+    [
+        Union[UnionData, UnionClass],
+        Union[UnionClass, UnionData],
+    ],
+)
+def test_class_path_union_mixture_dataclass_and_class(parser, union_type):
+    parser.add_argument("--union", type=union_type, enable_path=True)
+
+    value = {"class_path": f"{__name__}.UnionData", "init_args": {"data_a": 2, "data_b": "x"}}
+    cfg = parser.parse_args([f"--union={json.dumps(value)}"])
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.union, UnionData)
+    assert dataclasses.asdict(init.union) == {"data_a": 2, "data_b": "x"}
+    assert json_or_yaml_load(parser.dump(cfg))["union"] == value["init_args"]
+
+    value = {"class_path": f"{__name__}.UnionClass", "init_args": {"prm_1": 1.2, "prm_2": False}}
+    cfg = parser.parse_args([f"--union={json.dumps(value)}"])
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.union, UnionClass)
+    assert init.union.prm_1 == 1.2
+    assert json_or_yaml_load(parser.dump(cfg))["union"] == value
+
+
 # final classes tests
 
 
@@ -574,6 +755,18 @@ if annotated and pydantic_support > 1:
         cfg = parser.parse_args(["--n", "{}"])
         assert cfg.n == Namespace(a1=Namespace(a2=1))
 
+    class PingTask(pydantic.BaseModel):
+        type: Literal["ping"] = "ping"
+        attr: str = ""
+
+    class PongTask(pydantic.BaseModel):
+        type: Literal["pong"] = "pong"
+
+    PingPongTask = annotated[
+        Union[PingTask, PongTask],
+        pydantic.Field(discriminator="type"),
+    ]
+
 
 length = "length"
 if pydantic_support:
@@ -631,6 +824,17 @@ if pydantic_support:
         class PydanticAnnotatedField(pydantic.BaseModel):
             p1: annotated[int, pydantic.Field(default=2, ge=1, le=8)]  # type: ignore[valid-type]
 
+    class OptionalPydantic:
+        def __init__(self, a: Optional[PydanticModel] = None):
+            self.a = a
+
+    class NestedModel(pydantic.BaseModel):
+        inputs: List[str]
+        outputs: List[str]
+
+    class PydanticNestedDict(pydantic.BaseModel):
+        nested: Optional[Dict[str, NestedModel]] = None
+
 
 def none(x):
     return x
@@ -656,6 +860,8 @@ class TestPydantic:
         parser.add_argument("--model", type=PydanticSubModel, default=PydanticSubModel(p1="a"))
         cfg = parser.parse_args(["--model.p3=0.2"])
         assert Namespace(p1="a", p2=3, p3=0.2) == cfg.model
+        init = parser.instantiate_classes(cfg)
+        assert isinstance(init.model, PydanticSubModel)
 
     def test_field_default_factory(self, parser):
         parser.add_argument("--model", type=PydanticFieldFactory)
@@ -681,6 +887,18 @@ class TestPydantic:
             parser.parse_args(["--model.p1=0"])
         ctx.match("model.p1")
 
+    @pytest.mark.skipif(not (annotated and pydantic_support > 1), reason="Annotated is required")
+    def test_field_union_discriminator_dot_syntax(self, parser):
+        parser.add_argument("--model", type=PingPongTask)
+        cfg = parser.parse_args(["--model.type=pong"])
+        assert cfg.model == Namespace(type="pong")
+        init = parser.instantiate_classes(cfg)
+        assert isinstance(init.model, PongTask)
+        cfg = parser.parse_args(["--model.type=ping", "--model.attr=abc"])
+        assert cfg.model == Namespace(type="ping", attr="abc")
+        init = parser.instantiate_classes(cfg)
+        assert isinstance(init.model, PingTask)
+
     @pytest.mark.parametrize(
         ["valid_value", "invalid_value", "cast", "type_str"],
         [
@@ -705,7 +923,7 @@ class TestPydantic:
         parser.add_argument("--model", type=Model)
         cfg = parser.parse_args([f"--model.param={valid_value}"])
         assert cast(cfg.model.param) == valid_value
-        dump = yaml.safe_load(parser.dump(cfg))
+        dump = json_or_yaml_load(parser.dump(cfg))
         assert dump == {"model": {"param": valid_value}}
         with pytest.raises(ArgumentError) as ctx:
             parser.parse_args([f"--model.param={invalid_value}"])
@@ -721,11 +939,11 @@ class TestPydantic:
         parser.add_argument("--data", type=PydanticDataFieldInitFalse)
         help_str = get_parser_help(parser)
         assert "--data.p1" not in help_str
-        cfg = parser.parse_args(["--data", "{}"])
-        assert cfg.data == Namespace()
+        cfg = parser.parse_args([])
+        assert cfg == Namespace()
 
-        cfg = parser.instantiate_classes(cfg)
-        assert cfg.data.p1 == "-"
+        init = parser.instantiate_classes(cfg)
+        assert init.data.p1 == "-"
 
     def test_dataclass_stdlib_field(self, parser):
         parser.add_argument("--data", type=PydanticDataStdlibField)
@@ -748,6 +966,51 @@ class TestPydantic:
         cfg = parser.parse_args(["--data", '{"p3": {"p1": 1.0}}'])
         assert cfg.data == Namespace(p3=Namespace(p1=1.0, p2="-"))
 
+    def test_optional_pydantic_model(self, parser):
+        parser.add_argument("--b", type=OptionalPydantic)
+        parser.add_argument("--cfg", action="config")
+        cfg = parser.parse_args([f"--b={__name__}.OptionalPydantic"])
+        assert cfg.b.class_path == f"{__name__}.OptionalPydantic"
+        assert cfg.b.init_args == Namespace(a=None)
+        config = {
+            "b": {
+                "class_path": f"{__name__}.OptionalPydantic",
+                "init_args": {"a": {"p1": "x"}},
+            }
+        }
+        cfg = parser.parse_args([f"--cfg={json.dumps(config)}"])
+        assert cfg.b.class_path == f"{__name__}.OptionalPydantic"
+        assert cfg.b.init_args == Namespace(a=Namespace(p1="x", p2=3))
+
+    def test_nested_dict(self, parser):
+        parser.add_argument("--config", action="config")
+        parser.add_argument("--model", type=PydanticNestedDict)
+        model = {
+            "nested": {
+                "key": {
+                    "inputs": ["a", "b"],
+                    "outputs": ["x", "y"],
+                }
+            }
+        }
+        cfg = parser.parse_args(["--model", json.dumps(model)])
+        assert cfg.model.nested["key"] == Namespace(inputs=["a", "b"], outputs=["x", "y"])
+        init = parser.instantiate_classes(cfg)
+        assert isinstance(init.model, PydanticNestedDict)
+        assert isinstance(init.model.nested["key"], NestedModel)
+
+    def test_dashes_in_nested_dataclass(self):
+        class UnderscoresToDashesParser(ArgumentParser):
+            def add_argument(self, *args, **kwargs):
+                args = [arg.replace("_", "-") for arg in args]
+                return super().add_argument(*args, **kwargs)
+
+        parser = UnderscoresToDashesParser(parse_as_dict=False, default_env=True)
+        parser.add_class_arguments(NestedDefaultsD)
+        ns = parser.parse_args([])
+        cfg = parser.instantiate_classes(ns)
+        assert cfg.c_with_dash.field_with_dash == 5
+
 
 # attrs tests
 
@@ -774,6 +1037,21 @@ if attrs_support:
         def __attrs_post_init__(self):
             self.p1 = {}
 
+    @attrs.define
+    class AttrsSubField:
+        p1: str = "-"
+        p2: int = 0
+
+    @attrs.define
+    class AttrsWithNestedDefaultDataclass:
+        p1: float
+        subfield: AttrsSubField = attrs.field(factory=AttrsSubField)
+
+    @attrs.define
+    class AttrsWithNestedDataclassNoDefault:
+        p1: float
+        subfield: AttrsSubField
+
 
 @pytest.mark.skipif(not attrs_support, reason="attrs package is required")
 class TestAttrs:
@@ -804,9 +1082,19 @@ class TestAttrs:
         # not included in config object or its value is None.')
 
         parser.add_argument("--data", type=AttrsFieldInitFalse)
-        cfg = parser.parse_args(["--data", "{}"])
+        cfg = parser.parse_args([])
         help_str = get_parser_help(parser)
         assert "--data.p1" not in help_str
-        assert cfg.data == Namespace()
-        cfg = parser.instantiate_classes(cfg)
-        assert cfg.data.p1 == {}
+        assert cfg == Namespace()
+        init = parser.instantiate_classes(cfg)
+        assert init.data.p1 == {}
+
+    def test_nested_with_default(self, parser):
+        parser.add_argument("--data", type=AttrsWithNestedDefaultDataclass)
+        cfg = parser.parse_args(["--data.p1=1.23"])
+        assert cfg.data == Namespace(p1=1.23, subfield=Namespace(p1="-", p2=0))
+
+    def test_nested_without_default(self, parser):
+        parser.add_argument("--data", type=AttrsWithNestedDataclassNoDefault)
+        cfg = parser.parse_args(["--data.p1=1.23"])
+        assert cfg.data == Namespace(p1=1.23, subfield=Namespace(p1="-", p2=0))
