@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import dataclasses
 import os
 import pathlib
 from calendar import Calendar
@@ -13,13 +14,14 @@ import pytest
 
 from jsonargparse import (
     CLI,
-    ActionConfigFile,
     ActionJsonnet,
     ArgumentError,
     ArgumentParser,
-    LoggerProperty,
+    Namespace,
     Path,
     get_config_read_mode,
+    set_config_read_mode,
+    set_docstring_parse_options,
     set_url_support,
 )
 from jsonargparse._deprecated import (
@@ -28,22 +30,38 @@ from jsonargparse._deprecated import (
     ActionOperators,
     ActionPath,
     ActionPathList,
+    LoggerProperty,
     ParserError,
     deprecation_warning,
+    namespace_to_dict,
     shown_deprecation_warnings,
     usage_and_exit_error_handler,
 )
-from jsonargparse._optionals import jsonnet_support, url_support
+from jsonargparse._optionals import (
+    docstring_parser_support,
+    get_docstring_parse_options,
+    jsonnet_support,
+    pyyaml_available,
+    url_support,
+)
 from jsonargparse._util import argument_error
 from jsonargparse_tests.conftest import (
     get_parser_help,
     is_posix,
     skip_if_docstring_parser_unavailable,
+    skip_if_fsspec_unavailable,
     skip_if_requests_unavailable,
 )
+from jsonargparse_tests.test_dataclass_like import DataClassA
 from jsonargparse_tests.test_jsonnet import example_2_jsonnet
 
 
+@pytest.fixture(autouse=True)
+def no_pyyaml_skip():
+    if not pyyaml_available:
+        pytest.skip("pyyaml package is required")
+
+
 @pytest.fixture(autouse=True)
 def clear_shown_deprecation_warnings():
     yield
@@ -173,7 +191,13 @@ def test_ActionOperators():
 
 @skip_if_requests_unavailable
 def test_url_support_true():
-    assert "fr" == get_config_read_mode()
+    with catch_warnings(record=True) as w:
+        assert "fr" == get_config_read_mode()
+    assert_deprecation_warn(
+        w,
+        message="get_config_read_mode was deprecated",
+        code="get_config_read_mode()",
+    )
     with catch_warnings(record=True) as w:
         set_url_support(True)
     assert_deprecation_warn(
@@ -188,7 +212,13 @@ def test_url_support_true():
 
 @pytest.mark.skipif(url_support, reason="requests package should not be installed")
 def test_url_support_false():
-    assert "fr" == get_config_read_mode()
+    with catch_warnings(record=True) as w:
+        assert "fr" == get_config_read_mode()
+    assert_deprecation_warn(
+        w,
+        message="get_config_read_mode was deprecated",
+        code="get_config_read_mode()",
+    )
     with catch_warnings(record=True) as w:
         with pytest.raises(ImportError):
             set_url_support(True)
@@ -198,6 +228,27 @@ def test_url_support_false():
     assert "fr" == get_config_read_mode()
 
 
+@skip_if_fsspec_unavailable
+def test_set_config_read_mode():
+    with catch_warnings(record=True) as w:
+        assert "fr" == get_config_read_mode()
+    assert_deprecation_warn(
+        w,
+        message="get_config_read_mode was deprecated",
+        code="get_config_read_mode()",
+    )
+    with catch_warnings(record=True) as w:
+        set_config_read_mode(fsspec_enabled=True)
+    assert_deprecation_warn(
+        w,
+        message="set_config_read_mode was deprecated",
+        code="set_config_read_mode(fsspec_enabled=True)",
+    )
+    assert "fsr" == get_config_read_mode()
+    set_config_read_mode(fsspec_enabled=False)
+    assert "fr" == get_config_read_mode()
+
+
 def test_instantiate_subclasses():
     parser = ArgumentParser(exit_on_error=False)
     parser.add_argument("--cal", type=Calendar)
@@ -246,13 +297,27 @@ def test_multiple_functions_cli():
     assert isinstance(parser, ArgumentParser)
 
 
+class InheritsLoggerProperty(LoggerProperty):
+    pass
+
+
+def test_logger_property():
+    with catch_warnings(record=True) as w:
+        InheritsLoggerProperty()
+    assert_deprecation_warn(
+        w,
+        message="LoggerProperty was deprecated",
+        code="InheritsLoggerProperty()",
+    )
+
+
 def test_logger_property_none():
     with catch_warnings(record=True) as w:
-        LoggerProperty(logger=None)
+        ArgumentParser(logger=None)
     assert_deprecation_warn(
         w,
         message=" Setting the logger property to None was deprecated",
-        code="LoggerProperty(logger=None)",
+        code="ArgumentParser(logger=None)",
     )
 
 
@@ -329,7 +394,32 @@ def test_parse_as_dict(tmp_cwd):
     assert "{}\n" == parser.dump({})
     parser.save({}, "config.yaml")
     with open("config.yaml") as f:
-        assert "{}\n", f.read()
+        assert "{}\n" == f.read()
+
+
+def test_deprecated_skip_check_method(parser):
+    parser.add_argument("--key", type=int)
+    cfg = Namespace(key=1)
+    with catch_warnings(record=True) as w:
+        parser.check_config(cfg)
+    assert_deprecation_warn(
+        w,
+        message="ArgumentParser.check_config was deprecated",
+        code="parser.check_config(cfg)",
+    )
+
+
+def test_deprecated_dump_skip_check_parameter(parser):
+    parser.add_argument("--key", type=int)
+    cfg = Namespace(key="-")
+    with catch_warnings(record=True) as w:
+        dump = parser.dump(cfg, skip_check=True)
+    assert "-" in dump
+    assert_deprecation_warn(
+        w,
+        message="skip_check parameter was deprecated",
+        code="parser.dump(cfg, skip_check=True)",
+    )
 
 
 def test_ActionPath(tmp_cwd):
@@ -340,7 +430,7 @@ def test_ActionPath(tmp_cwd):
         output_file.write("file: " + rel_yaml_file + "\ndir: " + str(tmp_cwd) + "\n")
 
     parser = ArgumentParser(exit_on_error=False)
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     with catch_warnings(record=True) as w:
         parser.add_argument("--file", action=ActionPath(mode="fr"))
     assert_deprecation_warn(
@@ -517,6 +607,24 @@ def test_import_import_docstring_parse():
     assert import_docstring_parse is import_docstring_parser
 
 
+@skip_if_docstring_parser_unavailable
+def test_docstring_parse_options():
+    from docstring_parser import DocstringStyle
+
+    options = get_docstring_parse_options()
+    options["style"] = None
+
+    with catch_warnings(record=True) as w:
+        for style in [DocstringStyle.NUMPYDOC, DocstringStyle.GOOGLE]:
+            set_docstring_parse_options(style=style)
+            assert options["style"] == style
+    assert_deprecation_warn(
+        w,
+        message="set_docstring_parse_options was deprecated",
+        code="set_docstring_parse_options(style=style)",
+    )
+
+
 def test_import_from_deprecated():
     import jsonargparse.deprecated as deprecated
 
@@ -576,3 +684,45 @@ def test_action_jsonnet_ext_vars(parser):
     assert 9 == len(cfg.jsonnet["records"])
     assert "#8" == cfg.jsonnet["records"][-2]["ref"]
     assert 15.5 == cfg.jsonnet["records"][-2]["val"]
+
+
+def test_add_dataclass_arguments(parser, subtests):
+    with catch_warnings(record=True) as w:
+        parser.add_dataclass_arguments(DataClassA, "a", default=DataClassA(), title="CustomA title")
+    assert_deprecation_warn(
+        w,
+        message="add_dataclass_arguments was deprecated",
+        code='parser.add_dataclass_arguments(DataClassA, "a", default=DataClassA(), title="CustomA title")',
+    )
+
+    with subtests.test("get_defaults"):
+        cfg = parser.get_defaults()
+        assert dataclasses.asdict(DataClassA()) == cfg["a"].as_dict()
+        dump = __import__("yaml").safe_load(parser.dump(cfg))
+        assert dataclasses.asdict(DataClassA()) == dump["a"]
+
+    with subtests.test("instantiate_classes"):
+        init = parser.instantiate_classes(cfg)
+        assert isinstance(init["a"], DataClassA)
+
+    with subtests.test("docstrings in help"):
+        help_str = get_parser_help(parser)
+        if docstring_parser_support:
+            assert "CustomA title:" in help_str
+
+
+def test_namespace_to_dict():
+    ns = Namespace()
+    ns["w"] = 1
+    ns["x.y"] = 2
+    ns["x.z"] = 3
+    with catch_warnings(record=True) as w:
+        dic1 = namespace_to_dict(ns)
+    dic2 = ns.as_dict()
+    assert dic1 == dic2
+    assert dic1 is not dic2
+    assert_deprecation_warn(
+        w,
+        message="namespace_to_dict was deprecated",
+        code="dic1 = namespace_to_dict(ns)",
+    )
