@@ -1,14 +1,14 @@
 from __future__ import annotations
 
+import json
 from pathlib import Path
 from typing import Any, Dict, Generic, List, Optional, Tuple, TypeVar, Union
 from unittest.mock import patch
 
 import pytest
-import yaml
 
 from jsonargparse import (
-    ActionConfigFile,
+    ActionParser,
     ArgumentError,
     Namespace,
     lazy_instance,
@@ -20,6 +20,8 @@ from jsonargparse_tests.conftest import (
     capture_logs,
     get_parse_args_stdout,
     get_parser_help,
+    json_or_yaml_dump,
+    json_or_yaml_load,
     skip_if_docstring_parser_unavailable,
 )
 
@@ -81,7 +83,7 @@ class Class3(Class2):
 def test_add_class_failure_not_a_class(parser):
     with pytest.raises(ValueError) as ctx:
         parser.add_class_arguments("Not a class")
-    ctx.match('Expected "theclass" parameter to be a class')
+    ctx.match("Expected 'theclass' parameter to be a class")
 
 
 def test_add_class_failure_positional_without_type(parser):
@@ -94,14 +96,15 @@ def test_add_class_without_nesting(parser):
     parser.add_class_arguments(Class3)
 
     assert "Class3" in parser.groups
-    for key in "c3_a0 c3_a1 c3_a2 c3_a3 c3_a4 c3_a5 c3_a6 c3_a7 c3_a8 c1_a2 c1_a4 c1_a5".split():
+    for key in "c3_a0 c3_a1 c3_a2 c3_a3 c3_a4 c3_a5 c3_a6 c3_a7 c3_a8 c1_a2 c1_a3 c1_a4 c1_a5".split():
         assert _find_action(parser, key) is not None, f"{key} should be in parser but is not"
-    for key in ["c2_a0", "c1_a1", "c1_a3", "c0_a0"]:
+    for key in ["c2_a0", "c1_a1", "c0_a0"]:
         assert _find_action(parser, key) is None, f"{key} should not be in parser but is"
 
     cfg = parser.parse_args(["--c3_a0=0", "--c3_a3=true", "--c3_a4=a"], with_meta=False)
     assert cfg.as_dict() == {
         "c1_a2": 2.0,
+        "c1_a3": None,
         "c1_a4": 4,
         "c1_a5": "five",
         "c3_a0": 0,
@@ -135,12 +138,12 @@ def test_add_class_nested_as_group_false(parser):
     added_args = parser.add_class_arguments(Class3, "g", as_group=False)
 
     assert "g" not in parser.groups
-    assert 12 == len(added_args)
+    assert 13 == len(added_args)
     assert all(a.startswith("g.") for a in added_args)
 
-    for key in "c3_a0 c3_a1 c3_a2 c3_a3 c3_a4 c3_a5 c3_a6 c3_a7 c3_a8 c1_a2 c1_a4 c1_a5".split():
+    for key in "c3_a0 c3_a1 c3_a2 c3_a3 c3_a4 c3_a5 c3_a6 c3_a7 c3_a8 c1_a2 c1_a3 c1_a4 c1_a5".split():
         assert _find_action(parser, f"g.{key}") is not None, f"{key} should be in parser but is not"
-    for key in ["c2_a0", "c1_a1", "c1_a3", "c0_a0"]:
+    for key in ["c2_a0", "c1_a1", "c0_a0"]:
         assert _find_action(parser, f"g.{key}") is None, f"{key} should not be in parser but is"
 
     defaults = parser.get_defaults()
@@ -179,7 +182,7 @@ class NoParams:
 
 
 def test_add_class_without_parameters(parser):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_class_arguments(NoParams, "no_params")
 
     help_str = get_parser_help(parser)
@@ -192,8 +195,8 @@ def test_add_class_without_parameters(parser):
 
     config = {"no_params": {"class_path": f"{__name__}.NoParams"}}
     with pytest.raises(ArgumentError) as ctx:
-        parser.parse_args([f"--cfg={config}"])
-    ctx.match("'no_params' got an unexpected value")
+        parser.parse_args([f"--cfg={json.dumps(config)}"])
+    ctx.match("Group 'no_params' does not accept nested key 'class_path'")
 
 
 class NestedWithParams:
@@ -217,13 +220,13 @@ def test_add_class_nested_with_and_without_parameters(parser):
     assert isinstance(init.group.second, NestedWithoutParams)
 
 
-class NoValidParams:
-    def __init__(self, a0=None):
+class SkippedUnderscoreParam:
+    def __init__(self, _a0=None):
         pass
 
 
-def test_add_class_without_valid_parameters(parser):
-    assert [] == parser.add_class_arguments(NoValidParams)
+def test_add_class_skipped_underscore_parameter(parser):
+    assert [] == parser.add_class_arguments(SkippedUnderscoreParam)
 
 
 class WithNew:
@@ -242,12 +245,23 @@ def test_add_class_implemented_with_new(parser):
 
 class RequiredParams:
     def __init__(self, n: int, m: float):
-        pass
+        self.n = n
+        self.m = m
+
+
+def test_add_class_group_name_dash_required_parameters(parser):
+    parser.add_class_arguments(RequiredParams, "required-params")
+    assert "required-params" in parser.groups
+    cfg = parser.parse_args(["--required-params.n=6", "--required-params.m=0.9"])
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.required_params, RequiredParams)
+    assert init.required_params.n == 6
+    assert init.required_params.m == 0.9
 
 
 def test_add_class_with_required_parameters(parser):
     parser.add_class_arguments(RequiredParams, "model")
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
 
     for args in [
         [],
@@ -258,7 +272,7 @@ def test_add_class_with_required_parameters(parser):
         pytest.raises(ArgumentError, lambda: parser.parse_args(args))
 
     out = get_parse_args_stdout(parser, ["--model.m=0.1", "--print_config"])
-    assert "  n: null" in out
+    assert json_or_yaml_load(out) == {"model": {"n": None, "m": 0.1}}
 
     cfg = parser.parse_args([f"--config={out}", "--model.n=3"])
     assert cfg.model == Namespace(m=0.1, n=3)
@@ -341,7 +355,7 @@ def test_add_class_in_subcommand(parser, subparser):
 def test_add_class_group_config(parser, tmp_cwd):
     parser.add_class_arguments(Class0, "a")
     path = Path("a.yaml")
-    path.write_text("c0_a0: x")
+    path.write_text(json_or_yaml_dump({"c0_a0": "x"}))
     cfg = parser.parse_args(["--a=a.yaml"])
     assert str(cfg.a.pop("__path__")) == str(path)
     assert cfg.a == Namespace(c0_a0="x")
@@ -349,9 +363,8 @@ def test_add_class_group_config(parser, tmp_cwd):
 
 def test_add_class_group_config_not_found(parser, tmp_cwd):
     parser.add_class_arguments(Class0, "a")
-    with pytest.raises(ArgumentError) as ctx:
+    with pytest.raises(ArgumentError, match="Unable to load config .does_not_exist.yaml"):
         parser.parse_args(["--a=does_not_exist.yaml"])
-    ctx.match('Unable to load config "does_not_exist.yaml"')
 
 
 class WithDocstring:
@@ -419,25 +432,54 @@ def test_add_class_unmatched_default_type(parser):
     init = parser.instantiate_classes(cfg)
     assert init.cls.p2 == "deprecated"
     dump = parser.dump(cfg)
-    assert dump == "cls:\n  p1: x\n  p2: deprecated\n"
+    assert json_or_yaml_load(dump) == {"cls": {"p1": "x", "p2": "deprecated"}}
+
+
+class LeafClass:
+    def __init__(self, p1: int = 1, p2: str = "-"):
+        self.p1 = p1
+        self.p2 = p2
+
+
+def test_add_class_and_action_parser(parser, subparser):
+    subparser.add_class_arguments(LeafClass, "deep.leaf")
+    parser.add_argument("--nested", action=ActionParser(parser=subparser))
+    parser.add_argument("--config", action="config")
+
+    config = {
+        "nested": {
+            "deep": {
+                "leaf": {
+                    "p1": 2,
+                    "p2": "x",
+                }
+            }
+        }
+    }
+    cfg = parser.parse_args([f"--config={json.dumps(config)}"])
+    init = parser.instantiate_classes(cfg)
+    assert list(cfg.keys()) == ["nested.deep.leaf.p1", "nested.deep.leaf.p2", "config"]
+    assert list(init.keys()) == ["nested.deep.leaf", "config"]
+    assert isinstance(init.nested.deep.leaf, LeafClass)
+    assert init.nested.deep.leaf.p1 == 2
+    assert init.nested.deep.leaf.p2 == "x"
 
 
 # add_method_arguments tests
 
 
 class WithMethods:
-    def normal_method(self, a1="1", a2: float = 2.0, a3: bool = False, a4=None):
+    def normal_method(self, a1="1", a2: float = 2.0, a3: bool = False):
         """normal_method short description
 
         Args:
             a1: a1 description
             a2: a2 description
-            a4: a4 description
         """
         return a1
 
     @staticmethod
-    def static_method(a1: str, a2: float = 2.0, a3=None):
+    def static_method(a1: str, a2: float = 2.0, _a3=None):
         return a1
 
 
@@ -461,8 +503,7 @@ def test_add_method_normal_and_static(parser):
 
     for key in ["m.a1", "m.a2", "m.a3", "s.a1", "s.a2"]:
         assert _find_action(parser, key) is not None, f"{key} should be in parser but is not"
-    for key in ["m.a4", "s.a3"]:
-        assert _find_action(parser, key) is None, f"{key} should not be in parser but is"
+    assert _find_action(parser, "s._a3") is None, "s._a3 should not be in parser but is"
 
     cfg = parser.parse_args(["--m.a1=x", "--s.a1=y"], with_meta=False).as_dict()
     assert cfg == {"m": {"a1": "x", "a2": 2.0, "a3": False}, "s": {"a1": "y", "a2": 2.0}}
@@ -515,17 +556,16 @@ def test_add_function_arguments(parser):
 
     assert "func" in parser.groups
 
-    for key in ["a1", "a2", "a3"]:
+    for key in ["a1", "a2", "a3", "a4"]:
         assert _find_action(parser, key) is not None, f"{key} should be in parser but is not"
-    assert _find_action(parser, "a4") is None, "a4 should not be in parser but is"
 
     cfg = parser.parse_args(["--a1=x"], with_meta=False).as_dict()
-    assert cfg == {"a1": "x", "a2": 2.0, "a3": False}
+    assert cfg == {"a1": "x", "a2": 2.0, "a3": False, "a4": None}
     assert "x" == func(**cfg)
 
     if docstring_parser_support:
         assert "func short description" == parser.groups["func"].title
-        for key in ["a1", "a2"]:
+        for key in ["a1", "a2", "a4"]:
             assert f"{key} description" == _find_action(parser, key).help
 
 
@@ -593,6 +633,16 @@ def test_add_function_fail_untyped_false(parser):
     assert Namespace(a1=None, a2=None) == parser.parse_args([])
 
 
+def func_untyped_optional(a1: str, a2=None):
+    return a1
+
+
+def test_add_function_fail_untyped_true_untyped_optional(parser):
+    added_args = parser.add_function_arguments(func_untyped_optional, fail_untyped=True)
+    assert ["a1", "a2"] == added_args
+    assert Namespace(a1="x", a2=None) == parser.parse_args(["--a1=x"])
+
+
 def func_config(a1="1", a2: float = 2.0, a3: bool = False):
     return a1
 
@@ -602,13 +652,13 @@ def test_add_function_group_config(parser, tmp_cwd):
     parser.add_function_arguments(func, "func")
 
     cfg_path = Path("config.yaml")
-    cfg_path.write_text(yaml.dump({"a1": "one", "a3": True}))
+    cfg_path.write_text(json_or_yaml_dump({"a1": "one", "a3": True}))
 
     cfg = parser.parse_args([f"--func={cfg_path}"])
-    assert cfg.func == Namespace(a1="one", a2=2.0, a3=True)
+    assert cfg.func == Namespace(a1="one", a2=2.0, a3=True, a4=None)
 
     cfg = parser.parse_args(['--func={"a1": "ONE"}'])
-    assert cfg.func == Namespace(a1="ONE", a2=2.0, a3=False)
+    assert cfg.func == Namespace(a1="ONE", a2=2.0, a3=False, a4=None)
 
     with pytest.raises(ArgumentError) as ctx:
         parser.parse_args(['--func="""'])
@@ -616,18 +666,18 @@ def test_add_function_group_config(parser, tmp_cwd):
 
 
 def test_add_function_group_config_within_config(parser, tmp_cwd):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_function_arguments(func, "func")
 
     cfg_path = Path("subdir", "config.yaml")
     subcfg_path = Path("subsubdir", "func_config.yaml")
     Path("subdir", "subsubdir").mkdir(parents=True)
-    cfg_path.write_text(f"func: {subcfg_path}\n")
-    (cfg_path.parent / subcfg_path).write_text(yaml.dump({"a1": "one", "a3": True}))
+    cfg_path.write_text(json_or_yaml_dump({"func": str(subcfg_path)}))
+    (cfg_path.parent / subcfg_path).write_text(json_or_yaml_dump({"a1": "one", "a3": True}))
 
     cfg = parser.parse_args([f"--cfg={cfg_path}"])
     assert str(cfg.func.__path__) == str(subcfg_path)
-    assert strip_meta(cfg.func) == Namespace(a1="one", a2=2.0, a3=True)
+    assert strip_meta(cfg.func) == Namespace(a1="one", a2=2.0, a3=True, a4=None)
 
 
 def func_param_conflict(p1: int, cfg: dict):
@@ -635,7 +685,7 @@ def func_param_conflict(p1: int, cfg: dict):
 
 
 def test_add_function_param_conflict(parser):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     with pytest.raises(ValueError) as ctx:
         parser.add_function_arguments(func_param_conflict)
     ctx.match("Unable to add parameter 'cfg' from")
