@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import json
 import os
+import sys
 import textwrap
 import warnings
 from calendar import Calendar, HTMLCalendar, TextCalendar
@@ -9,26 +10,27 @@ from copy import deepcopy
 from dataclasses import dataclass
 from gzip import GzipFile
 from pathlib import Path
-from typing import Any, Dict, Iterable, List, Mapping, Optional, Union
+from typing import Any, Dict, Iterable, List, Mapping, Optional, Protocol, Union
 from unittest.mock import patch
 from uuid import NAMESPACE_OID
 
 import pytest
-import yaml
 
 from jsonargparse import (
-    ActionConfigFile,
     ArgumentError,
     ArgumentParser,
     Namespace,
     lazy_instance,
 )
+from jsonargparse._typehints import implements_protocol, is_instance_or_supports_protocol
 from jsonargparse.typing import final
 from jsonargparse_tests.conftest import (
     capture_logs,
     get_parse_args_stderr,
     get_parse_args_stdout,
     get_parser_help,
+    json_or_yaml_dump,
+    json_or_yaml_load,
     source_unavailable,
 )
 
@@ -40,7 +42,7 @@ def test_subclass_basics(parser, type):
         "init_args": {"firstweekday": 3},
     }
     parser.add_argument("--op", type=type)
-    cfg = parser.parse_args([f"--op={value}"])
+    cfg = parser.parse_args([f"--op={json.dumps(value)}"])
     init = parser.instantiate_classes(cfg)
     assert isinstance(init["op"], Calendar)
     assert 3 == init["op"].firstweekday
@@ -49,6 +51,24 @@ def test_subclass_basics(parser, type):
     assert init["op"] is None
 
 
+class BaseClassDefault:
+    def __init__(self, param: str = "base_default"):
+        self.param = param
+
+
+class SubClassDefault(BaseClassDefault):
+    def __init__(self, param: str = "sub_default"):
+        super().__init__(param=param)
+
+
+def test_subclass_defaults(parser):
+    parser.add_subclass_arguments(BaseClassDefault, "cls")
+    cfg = parser.parse_args(["--cls=BaseClassDefault"])
+    assert cfg.cls.init_args.param == "base_default"
+    cfg = parser.parse_args(["--cls=SubClassDefault"])
+    assert cfg.cls.init_args.param == "sub_default"
+
+
 def test_subclass_init_args_in_subcommand(parser, subparser):
     subparser.add_subclass_arguments(Calendar, "cal", default=lazy_instance(Calendar))
     subcommands = parser.add_subcommands()
@@ -156,7 +176,7 @@ def test_subclass_union_help(parser):
     help_str = get_parser_help(parser)
     assert "Show the help for the given subclass of Calendar" in help_str
     help_str = get_parse_args_stdout(parser, ["--op.help", "TextCalendar"])
-    assert "--op.init_args.firstweekday" in help_str
+    assert "--op.firstweekday" in help_str
 
 
 class DefaultsDisabled:
@@ -204,8 +224,8 @@ def func_subclass_untyped(c1: Union[int, UntypedParams]):
 def test_subclass_allow_untyped_parameters_help(parser):
     parser.add_function_arguments(func_subclass_untyped, fail_untyped=False)
     help_str = get_parse_args_stdout(parser, [f"--c1.help={__name__}.UntypedParams"])
-    assert "--c1.init_args.a1 A1" in help_str
-    assert "--c1.init_args.a2 A2" in help_str
+    assert "--c1.a1 A1" in help_str
+    assert "--c1.a2 A2" in help_str
 
 
 class MergeInitArgs(Calendar):
@@ -214,7 +234,7 @@ class MergeInitArgs(Calendar):
 
 
 def test_subclass_merge_init_args_global_config(parser):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_argument("--cal", type=Calendar)
 
     config1 = {
@@ -234,7 +254,7 @@ def test_subclass_merge_init_args_global_config(parser):
     expected = deepcopy(config1["cal"])
     expected["init_args"].update(config2["cal"]["init_args"])
 
-    cfg = parser.parse_args([f"--cfg={config1}", f"--cfg={config2}"])
+    cfg = parser.parse_args([f"--cfg={json.dumps(config1)}", f"--cfg={json.dumps(config2)}"])
     assert cfg.cal.as_dict() == expected
 
 
@@ -246,22 +266,15 @@ def test_subclass_init_args_without_class_path(parser):
     assert cfg.cal3.init_args == Namespace(firstweekday=5)
 
 
-def test_subclass_init_args_without_class_path_error(parser):
-    parser.add_subclass_arguments(Calendar, "cal1")
-    with pytest.raises(ArgumentError) as ctx:
-        parser.parse_args(["--cal1.init_args.firstweekday=4"])
-    ctx.match("class path given previously")
-
-
 def test_subclass_init_args_without_class_path_dict(parser):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_argument("--cal", type=Calendar)
     config = {"cal": {"class_path": "TextCalendar", "init_args": {"firstweekday": 2}}}
 
-    cfg = parser.parse_args([f"--cfg={config}", "--cal={'init_args': {'firstweekday': 3}}"])
+    cfg = parser.parse_args([f"--cfg={json.dumps(config)}", '--cal={"init_args": {"firstweekday": 3}}'])
     assert cfg.cal.init_args == Namespace(firstweekday=3)
 
-    cfg = parser.parse_args([f"--cfg={config}", "--cal={'firstweekday': 4}"])
+    cfg = parser.parse_args([f"--cfg={json.dumps(config)}", '--cal={"firstweekday": 4}'])
     assert cfg.cal.init_args == Namespace(firstweekday=4)
 
 
@@ -288,9 +301,8 @@ def test_subclass_with_default_config_files(parser, tmp_cwd, logger, subtests):
         assert cfg.data.cal.as_dict() == config
 
     with subtests.test("dump"):
-        dump = parser.dump(cfg)
-        assert "class_path: calendar.Calendar\n" in dump
-        assert "firstweekday: 3\n" in dump
+        dump = json_or_yaml_load(parser.dump(cfg))
+        assert dump["data"]["cal"] == {"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}
 
     with subtests.test("disable defaults"):
         cfg = parser.parse_args(["--data.cal.class_path=calendar.Calendar"], defaults=False)
@@ -311,10 +323,10 @@ class DefaultConfigSubcommands:
 
 def test_subclass_in_subcommand_with_global_default_config_file(parser, subparser, tmp_cwd):
     default_path = Path("default.yaml")
-    default_path.write_text("fit:\n  model:\n    foo: 123")
+    default_path.write_text(json_or_yaml_dump({"fit": {"model": {"foo": 123}}}))
 
     parser.default_config_files = [default_path]
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
     subcommands = parser.add_subcommands()
 
     subparser.add_class_arguments(DefaultConfigSubcommands, nested_key="model")
@@ -328,6 +340,77 @@ def test_subclass_in_subcommand_with_global_default_config_file(parser, subparse
     assert cfg.fit.model.foo == 123
 
 
+# function instantiators
+
+
+class ClassMethodInstantiator:
+    def __init__(self, p1: int = 1, p2: bool = False):
+        self.p1 = p1
+        self.p2 = p2
+
+    @classmethod
+    def from_p1(cls, p1: int) -> "ClassMethodInstantiator":
+        return ClassMethodInstantiator(p1)
+
+
+def test_class_method_instantiator(parser):
+    parser.add_argument("--cls", type=ClassMethodInstantiator)
+    cfg = parser.parse_args([f"--cls={__name__}.ClassMethodInstantiator.from_p1", "--cls.p1=2"])
+    assert cfg.cls.class_path == f"{__name__}.ClassMethodInstantiator.from_p1"
+    assert cfg.cls.init_args == Namespace(p1=2)
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.cls, ClassMethodInstantiator)
+    assert init.cls.p1 == 2
+    assert init.cls.p2 is False
+
+    with pytest.raises(ArgumentError) as ctx:
+        parser.parse_args([f"--cls={__name__}.ClassMethodInstantiator.from_p1"])
+    ctx.match('Key "p1" is required')
+    with pytest.raises(ArgumentError) as ctx:
+        parser.parse_args([f"--cls={__name__}.ClassMethodInstantiator.from_p1", "--cls.p1=2", "--cls.p3=-"])
+    ctx.match("Key 'p3' is not expected")
+
+
+class FunctionInstantiator:
+    def __init__(self, p1: float = 0.0, p2: str = "-"):
+        self.p1 = p1
+        self.p2 = p2
+
+
+def function_instantiator(p2: str) -> FunctionInstantiator:
+    return FunctionInstantiator(1.0, p2)
+
+
+def test_function_instantiator(parser):
+    parser.add_argument("--cls", type=FunctionInstantiator)
+    cfg = parser.parse_args([f"--cls={__name__}.function_instantiator", "--cls.p2=y"])
+    assert cfg.cls.class_path == f"{__name__}.function_instantiator"
+    assert cfg.cls.init_args == Namespace(p2="y")
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.cls, FunctionInstantiator)
+    assert init.cls.p1 == 1.0
+    assert init.cls.p2 == "y"
+
+    with pytest.raises(ArgumentError) as ctx:
+        parser.parse_args([f"--cls={__name__}.function_instantiator", "--cls.p2=y", "--cls.p3=x"])
+    ctx.match("Key 'p3' is not expected")
+
+
+def function_undefined_return(p1: int) -> "Undefined":  # type: ignore[name-defined]  # noqa: F821
+    return FunctionInstantiator(p1, "x")
+
+
+def test_instantiator_undefined_return(parser, logger):
+    parser.logger = logger
+    parser.add_argument("--cls", type=FunctionInstantiator)
+    with capture_logs(logger) as logs, pytest.raises(ArgumentError) as ctx:
+        parser.parse_args([f"--cls={__name__}.function_undefined_return", "--cls.p1=2"])
+    ctx.match("function_undefined_return does not correspond to a subclass of")
+    assert "function_undefined_return does not correspond to a subclass of" in logs.getvalue()
+    if sys.version_info >= (3, 9):
+        assert "Unable to evaluate types for" in logs.getvalue()
+
+
 # importable instances
 
 
@@ -342,8 +425,8 @@ def test_importable_instances(parser):
     parser.add_argument("--dtype", type=dtype)
     cfg = parser.parse_args([f"--dtype={__name__}.float32"])
     assert cfg.dtype is float32
-    dump = parser.dump(cfg)
-    assert dump.strip() == f"dtype: {__name__}.float32"
+    dump = json_or_yaml_load(parser.dump(cfg))
+    assert dump == {"dtype": f"{__name__}.float32"}
 
 
 # custom instantiation tests
@@ -414,6 +497,21 @@ def test_custom_instantiation_replace(parser):
     assert list(parser._instantiators.values())[0] is second_instantiator
 
 
+class CustomInstantiationNested:
+    def __init__(self, sub: CustomInstantiationBase):
+        self.sub = sub
+
+
+def test_custom_instantiation_nested(parser):
+    parser.add_argument("--cls", type=CustomInstantiationNested)
+    parser.add_instantiator(instantiator("nested"), CustomInstantiationBase, subclasses=True)
+    cfg = parser.parse_args(["--cls=CustomInstantiationNested", "--cls.sub=CustomInstantiationSub"])
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.cls, CustomInstantiationNested)
+    assert isinstance(init.cls.sub, CustomInstantiationSub)
+    assert init.cls.sub.call == "nested"
+
+
 # environment tests
 
 
@@ -432,7 +530,7 @@ def test_subclass_env_config(parser):
     parser.env_prefix = "APP"
     parser.default_env = True
     parser.add_argument("--cal", type=Calendar)
-    env = {"APP_CAL": "{'class_path': 'TextCalendar', 'init_args': {'firstweekday': 4}}"}
+    env = {"APP_CAL": '{"class_path": "TextCalendar", "init_args": {"firstweekday": 4}}'}
     with patch.dict(os.environ, env):
         cfg = parser.parse_env()
     assert cfg.cal == Namespace(class_path="calendar.TextCalendar", init_args=Namespace(firstweekday=4))
@@ -465,11 +563,15 @@ def test_subclass_nested_parse(parser, prefix):
 
 def test_subclass_nested_help(parser):
     parser.add_argument("--op", type=Nested)
-    help_str = get_parse_args_stdout(parser, [f"--op.help={__name__}.Nested", "--op.init_args.cal.help=TextCalendar"])
-    assert "--op.init_args.cal.init_args.firstweekday" in help_str
+    help_str = get_parse_args_stdout(parser, [f"--op.help={__name__}.Nested", "--op.cal.help=TextCalendar"])
+    assert "Help for --op.cal.help=calendar.TextCalendar" in help_str
+    assert "--op.cal.firstweekday" in help_str
+    help_str = get_parse_args_stdout(parser, [f"--op.help={__name__}.Nested", "--op.cal.help"])
+    assert "Help for --op.cal.help=calendar.Calendar" in help_str
+    assert "--op.cal.firstweekday" in help_str
 
     with pytest.raises(ArgumentError) as ctx:
-        parser.parse_args([f"--op.help={__name__}.Nested", "--op.init_args.p1=1"])
+        parser.parse_args([f"--op.help={__name__}.Nested", "--op.p1=1"])
     ctx.match("Expected a nested --\\*.help option")
 
 
@@ -484,22 +586,27 @@ class RequiredParamModel:
 
 
 def test_subclass_required_parameter_with_default_config_files(parser, tmp_cwd):
-    defaults = f"""model:
-      sub_module:
-        class_path: {__name__}.RequiredParamSubModule
-        init_args:
-          p1: 4
-          p2: 5
-    """
+    defaults = {
+        "model": {
+            "sub_module": {
+                "class_path": f"{__name__}.RequiredParamSubModule",
+                "init_args": {
+                    "p1": 4,
+                    "p2": 5,
+                },
+            },
+        },
+    }
     defaults_path = Path("defaults.yaml")
-    defaults_path.write_text(defaults)
+    defaults_path.write_text(json_or_yaml_dump(defaults))
 
     parser.default_config_files = [defaults_path]
     parser.add_class_arguments(RequiredParamModel, "model")
 
     cfg = parser.parse_args(["--model.sub_module.init_args.p2=7"])
 
-    expected = yaml.safe_load(defaults.replace("p2: 5", "p2: 7"))["model"]
+    expected = defaults["model"]
+    expected["sub_module"]["init_args"]["p2"] = 7
     expected["sub_module"]["init_args"]["p3"] = 3
     assert cfg.model.as_dict() == expected
 
@@ -516,7 +623,8 @@ def test_subclass_class_name_parse(parser):
 def test_subclass_class_name_help(parser):
     parser.add_argument("--op", type=Union[Calendar, GzipFile, None])
     help_str = get_parse_args_stdout(parser, ["--op.help=GzipFile"])
-    assert "--op.init_args.compresslevel" in help_str
+    assert "Help for --op.help=gzip.GzipFile" in help_str
+    assert "--op.compresslevel" in help_str
 
 
 class LocaleTextCalendar(Calendar):
@@ -557,7 +665,7 @@ def test_subclass_class_name_then_invalid_init_args(parser):
     parser.add_argument("--op", type=Union[Calendar, GzipFile])
     with pytest.raises(ArgumentError) as ctx:
         parser.parse_args(["--op=TextCalendar", "--op=GzipFile", "--op.firstweekday=2"])
-    ctx.match('No action for key "firstweekday"')
+    ctx.match("Key 'firstweekday' is not expected")
 
 
 # dict parameter tests
@@ -640,28 +748,31 @@ class Model:
 
 
 def test_subclass_dict_parameter_deep(parser):
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_class_arguments(Model, "model")
 
-    config = f"""model:
-      encoder:
-        class_path: {__name__}.Network
-        init_args:
-          some_dict:
-            a: 1
-          sub_network:
-            class_path: {__name__}.Network
-            init_args:
-              some_dict:
-                b: 2
-              sub_network:
-                class_path: {__name__}.Module
-    """
+    config = {
+        "model": {
+            "encoder": {
+                "class_path": f"{__name__}.Network",
+                "init_args": {
+                    "some_dict": {"a": 1},
+                    "sub_network": {
+                        "class_path": f"{__name__}.Network",
+                        "init_args": {
+                            "some_dict": {"b": 2},
+                            "sub_network": {"class_path": f"{__name__}.Module"},
+                        },
+                    },
+                },
+            },
+        },
+    }
 
-    cfg = parser.parse_args([f"--cfg={config}"])
+    cfg = parser.parse_args([f"--cfg={json_or_yaml_dump(config)}"])
     assert cfg.model.encoder.init_args.some_dict == {"a": 1}
     assert cfg.model.encoder.init_args.sub_network.init_args.some_dict == {"b": 2}
-    assert cfg.model.as_dict() == yaml.safe_load(config)["model"]
+    assert cfg.model.as_dict() == config["model"]
 
 
 # list append tests
@@ -672,12 +783,14 @@ class ListAppend:
         pass
 
 
-def test_subclass_list_append_single(parser):
-    parser.add_argument("--val", type=Union[ListAppend, List[ListAppend]])
+@pytest.mark.parametrize("list_type", [List, Iterable])
+def test_subclass_list_append_single(parser, list_type):
+    parser.add_argument("--val", type=Union[ListAppend, list_type[ListAppend]])
     cfg = parser.parse_args([f"--val+={__name__}.ListAppend", "--val.p1=1", "--val.p2=2", "--val.p1=3"])
     assert cfg.val == [Namespace(class_path=f"{__name__}.ListAppend", init_args=Namespace(p1=3, p2=2))]
     cfg = parser.parse_args(["--val+=ListAppend", "--val.p2=2", "--val.p1=1"])
     assert cfg.val == [Namespace(class_path=f"{__name__}.ListAppend", init_args=Namespace(p1=1, p2=2))]
+    assert " --val+ " in get_parser_help(parser)
 
 
 @final
@@ -716,7 +829,7 @@ def test_subclass_list_append_multiple(parser):
     )
     assert ["calendar.Calendar", "calendar.TextCalendar"] == [x.class_path for x in cfg.a.cals]
     assert [3, 1] == [x.init_args.firstweekday for x in cfg.a.cals]
-    cfg = parser.parse_args([f"--a={cfg.a.as_dict()}", "--a.cals.firstweekday=4"])
+    cfg = parser.parse_args([f"--a={json.dumps(cfg.a.as_dict())}", "--a.cals.firstweekday=4"])
     assert Namespace(firstweekday=4) == cfg.a.cals[-1].init_args
     args = ["--a.cals+=Invalid", "--a.cals+=TextCalendar"]
     pytest.raises(ArgumentError, lambda: parser.parse_args(args))
@@ -738,7 +851,7 @@ def test_subcommand_subclass_list_append_multiple(parser, subparser):
     )
     assert ["calendar.Calendar", "calendar.TextCalendar"] == [x.class_path for x in cfg.cmd.a.cals]
     assert [3, 1] == [x.init_args.firstweekday for x in cfg.cmd.a.cals]
-    cfg = parser.parse_args(["cmd", f"--a={cfg.cmd.a.as_dict()}", "--a.cals.firstweekday=4"])
+    cfg = parser.parse_args(["cmd", f"--a={json.dumps(cfg.cmd.a.as_dict())}", "--a.cals.firstweekday=4"])
     assert Namespace(firstweekday=4) == cfg.cmd.a.cals[-1].init_args
     args = ["cmd", "--a.cals+=Invalid", "--a.cals+=TextCalendar"]
     pytest.raises(ArgumentError, lambda: parser.parse_args(args))
@@ -770,7 +883,7 @@ def test_type_any_subclasses(parser):
         },
     }
 
-    cfg = parser.parse_args([f"--any={value}"])
+    cfg = parser.parse_args([f"--any={json.dumps(value)}"])
     init = parser.instantiate_classes(cfg)
     assert isinstance(init.any, AnySubclasses)
     assert isinstance(init.any.cal1, TextCalendar)
@@ -779,7 +892,7 @@ def test_type_any_subclasses(parser):
     assert 2 == init.any.cal2.firstweekday
 
     value["init_args"]["cal2"]["class_path"] = "does.not.exist"
-    cfg = parser.parse_args([f"--any={value}"])
+    cfg = parser.parse_args([f"--any={json.dumps(value)}"])
     assert isinstance(cfg.any.init_args.cal1, Namespace)
     assert isinstance(cfg.any.init_args.cal2, dict)
     init = parser.instantiate_classes(cfg)
@@ -790,7 +903,7 @@ def test_type_any_subclasses(parser):
     assert 2 == init.any.cal2["init_args"]["firstweekday"]
 
     value["init_args"]["cal1"]["class_path"] = "does.not.exist"
-    cfg = parser.parse_args([f"--any={value}"])
+    cfg = parser.parse_args([f"--any={json.dumps(value)}"])
     assert isinstance(cfg.any, dict)
 
 
@@ -807,7 +920,7 @@ def test_type_any_list_of_subclasses(parser):
         },
     ]
 
-    cfg = parser.parse_args([f"--any={value}"])
+    cfg = parser.parse_args([f"--any={json.dumps(value)}"])
     init = parser.instantiate_classes(cfg)
     assert isinstance(init.any, list)
     assert 2 == len(init.any)
@@ -830,7 +943,7 @@ def test_type_any_dict_of_subclasses(parser):
         },
     }
 
-    cfg = parser.parse_args([f"--any={value}"])
+    cfg = parser.parse_args([f"--any={json.dumps(value)}"])
     init = parser.instantiate_classes(cfg)
     assert isinstance(init.any, dict)
     assert 2 == len(init.any)
@@ -949,12 +1062,12 @@ class OverrideSub2(OverrideBase):
 
 def test_subclass_discard_init_args_config_with_default(parser, logger):
     parser.logger = logger
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_argument("--s", type=OverrideBase, default=lazy_instance(OverrideSub1, s1="v1"))
 
     config = {"s": {"class_path": "OverrideSub2", "init_args": {"s2": "v2"}}}
     with capture_logs(logger) as logs:
-        cfg = parser.parse_args([f"--cfg={config}"])
+        cfg = parser.parse_args([f"--cfg={json.dumps(config)}"])
 
     assert "discarding init_args: {'s1': 'v1'}" in logs.getvalue()
     assert cfg.s.class_path == f"{__name__}.OverrideSub2"
@@ -985,7 +1098,9 @@ def test_subclass_discard_init_args_with_default_config_files(parser, tmp_cwd, l
         cfg = parser.parse_args(['--cal={"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}'])
     assert "discarding init_args: {'param': '1'}" in logs.getvalue()
     assert cfg.cal.init_args == Namespace(firstweekday=3)
-    assert type(parser.instantiate_classes(cfg).cal) is Calendar
+    with capture_logs(logger) as logs:
+        assert type(parser.instantiate_classes(cfg).cal) is Calendar
+    assert logs.getvalue()
 
 
 class Arch:
@@ -1015,7 +1130,7 @@ def test_subclass_subcommand_set_defaults_discard_init_args(parser, subparser, l
     subcommands.add_subcommand("fit", subparser)
 
     with capture_logs(logger) as logs:
-        cfg = parser.parse_args(["fit", f"--arch={value}"])
+        cfg = parser.parse_args(["fit", f"--arch={json.dumps(value)}"])
     assert "discarding init_args: {'b': 3}" in logs.getvalue()
     assert cfg.fit.arch.as_dict() == value
 
@@ -1043,7 +1158,7 @@ class ConfigDiscardMain:
 @pytest.mark.parametrize("method", ["class", "subclass"])
 def test_discard_init_args_config_nested(parser, logger, tmp_cwd, method):
     parser.logger = logger
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
 
     subconfig = {
         "sub": {
@@ -1065,12 +1180,14 @@ def test_discard_init_args_config_nested(parser, logger, tmp_cwd, method):
         parser.set_defaults(main=lazy_instance(ConfigDiscardMain))
 
     config_path = Path("config.yaml")
-    config_path.write_text(yaml.safe_dump(config))
+    config_path.write_text(json_or_yaml_dump(config))
 
     with capture_logs(logger) as logs:
         cfg = parser.parse_args([f"--cfg={config_path}"])
     assert "discarding init_args: {'s1': 'x'}" in logs.getvalue()
-    init = parser.instantiate_classes(cfg)
+    with capture_logs(logger) as logs:
+        init = parser.instantiate_classes(cfg)
+    assert logs.getvalue()
     assert isinstance(init.main, ConfigDiscardMain)
     assert isinstance(init.main.sub, ConfigDiscardSub2)
 
@@ -1097,7 +1214,7 @@ class DictDiscardMain:
 
 def test_subclass_discard_init_args_dict_looks_like_subclass(parser, logger, tmp_cwd):
     parser.logger = logger
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_subclass_arguments(DictDiscardMain, "main")
     parser.set_defaults(main=lazy_instance(DictDiscardMain))
 
@@ -1116,12 +1233,14 @@ def test_subclass_discard_init_args_dict_looks_like_subclass(parser, logger, tmp
             }
         }
         config_paths[c] = Path(f"config{c}.yaml")
-        config_paths[c].write_text(yaml.safe_dump(configs[c]))
+        config_paths[c].write_text(json_or_yaml_dump(configs[c]))
 
     with capture_logs(logger) as logs:
         cfg = parser.parse_args([f"--cfg={config_paths[1]}", f"--cfg={config_paths[2]}"])
     assert "discarding init_args: {'s1': 1}" in logs.getvalue()
-    init = parser.instantiate_classes(cfg)
+    with capture_logs(logger) as logs:
+        init = parser.instantiate_classes(cfg)
+    assert logs.getvalue()
     assert isinstance(init.main, DictDiscardMain)
     assert isinstance(init.main.sub, dict)
     assert init.main.sub["init_args"]["s2"] == 2
@@ -1153,7 +1272,7 @@ def test_subclass_unresolved_parameters(parser, subtests):
         dict_kwargs={"p3": 7.0, "p4": "x"},
     )
 
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_argument("--cls", type=UnresolvedParams)
 
     with subtests.test("args"):
@@ -1163,15 +1282,15 @@ def test_subclass_unresolved_parameters(parser, subtests):
         assert cfg.cls.dict_kwargs == expected.dict_kwargs
 
     with subtests.test("config"):
-        cfg = parser.parse_args([f"--cfg={config}"])
+        cfg = parser.parse_args([f"--cfg={json.dumps(config)}"])
         assert cfg.cls == expected
         init = parser.instantiate_classes(cfg)
         assert isinstance(init.cls, UnresolvedParams)
         assert init.cls.kwargs == expected.dict_kwargs
 
     with subtests.test("print_config"):
-        out = get_parse_args_stdout(parser, [f"--cfg={config}", "--print_config"])
-        data = yaml.safe_load(out)["cls"]
+        out = get_parse_args_stdout(parser, [f"--cfg={json.dumps(config)}", "--print_config"])
+        data = json_or_yaml_load(out)["cls"]
         assert data == expected.as_dict()
 
     with subtests.test("invalid dict_kwargs"):
@@ -1203,13 +1322,13 @@ def test_subclass_unresolved_parameters_name_clash(parser):
 def test_add_subclass_failure_not_a_class(parser):
     with pytest.raises(ValueError) as ctx:
         parser.add_subclass_arguments(NAMESPACE_OID, "oid")
-    ctx.match("Expected 'baseclass' argument to be a class or a tuple of classes")
+    ctx.match("Expected 'baseclass' to be a subclass type or a tuple of subclass types")
 
 
 def test_add_subclass_failure_empty_tuple(parser):
     with pytest.raises(ValueError) as ctx:
         parser.add_subclass_arguments((), "cls")
-    ctx.match("Expected 'baseclass' argument to be a class or a tuple of classes")
+    ctx.match("Expected 'baseclass' to be a subclass type or a tuple of subclass types")
 
 
 def test_add_subclass_lazy_default(parser):
@@ -1218,11 +1337,10 @@ def test_add_subclass_lazy_default(parser):
     assert cfg.cal.class_path == "calendar.Calendar"
     assert cfg.cal.init_args.firstweekday == 4
 
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
     parser.set_defaults({"cal": lazy_instance(Calendar, firstweekday=5)})
     out = get_parse_args_stdout(parser, ["--print_config"])
-    assert "class_path: calendar.Calendar" in out
-    assert "firstweekday: 5" in out
+    assert json_or_yaml_load(out)["cal"] == {"class_path": "calendar.Calendar", "init_args": {"firstweekday": 5}}
 
     help_str = get_parser_help(parser)
     assert "'init_args': {'firstweekday': 5}" in help_str
@@ -1254,7 +1372,7 @@ def test_add_subclass_tuple(parser):
     assert isinstance(init.c, TupleBaseB)
 
     help_str = get_parse_args_stdout(parser, [f"--c.help={__name__}.TupleBaseB"])
-    assert "--c.init_args.b1" in help_str
+    assert "--c.b1 B1" in help_str
 
 
 def test_add_subclass_required_group(parser):
@@ -1262,15 +1380,49 @@ def test_add_subclass_required_group(parser):
     parser.add_subclass_arguments(Calendar, "cal", required=True)
     pytest.raises(ArgumentError, lambda: parser.parse_args([]))
     help_str = get_parser_help(parser)
-    assert "[-h] [--cal.help CLASS_PATH_OR_NAME] --cal " in help_str
+    assert "[-h] [--cal.help [CLASS_PATH_OR_NAME]] --cal " in help_str
 
 
 def test_add_subclass_not_required_group(parser):
     parser.add_subclass_arguments(Calendar, "cal", required=False)
     cfg = parser.parse_args([])
-    assert cfg == Namespace()
+    assert cfg == Namespace(cal=None)
     init = parser.instantiate_classes(cfg)
-    assert init == Namespace()
+    assert init == Namespace(cal=None)
+
+
+class ListUnionA:
+    def __init__(self, pa1: int):
+        self.pa1 = pa1
+
+
+class ListUnionB:
+    def __init__(self, pb1: str, pb2: float):
+        self.pb1 = pb1
+        self.pb2 = pb2
+
+
+def test_add_subclass_list_of_union(parser):
+    parser.add_argument("--config", action="config")
+    parser.add_subclass_arguments(
+        baseclass=(ListUnionA, ListUnionB, List[Union[ListUnionA, ListUnionB]]),
+        nested_key="subclass",
+    )
+    config = {
+        "subclass": [
+            {
+                "class_path": f"{__name__}.ListUnionB",
+                "init_args": {
+                    "pb1": "x",
+                    "pb2": 0.5,
+                },
+            }
+        ]
+    }
+    cfg = parser.parse_args([f"--config={json.dumps(config)}"])
+    assert cfg.as_dict()["subclass"] == config["subclass"]
+    help_str = get_parser_help(parser)
+    assert "Show the help for the given subclass of {ListUnionA,ListUnionB}" in help_str
 
 
 # instance defaults tests
@@ -1304,7 +1456,166 @@ def test_subclass_signature_instance_default(parser):
     with warnings.catch_warnings(record=True) as w:
         dump = parser.dump(cfg)
     assert "Unable to serialize instance" in str(w[0].message)
-    assert "cal: Unable to serialize instance <calendar.Calendar " in dump
+    assert "Unable to serialize instance <calendar.Calendar " in dump
+
+
+# protocol tests
+
+
+class Interface(Protocol):
+    def predict(self, items: List[float]) -> List[float]: ...
+
+
+class ImplementsInterface:
+    def __init__(self, batch_size: int):
+        self.batch_size = batch_size
+
+    def predict(self, items: List[float]) -> List[float]:
+        return items
+
+
+class SubclassImplementsInterface(Interface):
+    def __init__(self, max_items: int):
+        self.max_items = max_items
+
+    def predict(self, items: List[float]) -> List[float]:
+        return items
+
+
+class NotImplementsInterface1:
+    def predict(self, items: str) -> List[float]:
+        return []
+
+
+class NotImplementsInterface2:
+    def predict(self, items: List[float], extra: int) -> List[float]:
+        return items
+
+
+class NotImplementsInterface3:
+    def predict(self, items: List[float]) -> None:
+        return
+
+
+@pytest.mark.parametrize(
+    "expected, value",
+    [
+        (True, ImplementsInterface),
+        (True, SubclassImplementsInterface),
+        (False, ImplementsInterface(1)),
+        (False, NotImplementsInterface1),
+        (False, NotImplementsInterface2),
+        (False, NotImplementsInterface3),
+        (False, object),
+    ],
+)
+def test_implements_protocol(expected, value):
+    assert implements_protocol(value, Interface) is expected
+
+
+@pytest.mark.parametrize(
+    "expected, value",
+    [
+        (False, ImplementsInterface),
+        (True, ImplementsInterface(1)),
+        (False, NotImplementsInterface1()),
+        (False, object),
+    ],
+)
+def test_is_instance_or_supports_protocol(expected, value):
+    assert is_instance_or_supports_protocol(value, Interface) is expected
+
+
+def test_parse_implements_protocol(parser):
+    parser.add_argument("--cls", type=Interface)
+    cfg = parser.parse_args([f"--cls={__name__}.ImplementsInterface", "--cls.batch_size=5"])
+    assert cfg.cls.class_path == f"{__name__}.ImplementsInterface"
+    assert cfg.cls.init_args == Namespace(batch_size=5)
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.cls, ImplementsInterface)
+    assert init.cls.batch_size == 5
+    assert init.cls.predict([1.0, 2.0]) == [1.0, 2.0]
+
+    help_str = get_parser_help(parser)
+    assert "known subclasses:" in help_str
+    assert f"{__name__}.SubclassImplementsInterface" in help_str
+    help_str = get_parse_args_stdout(parser, ["--cls.help=SubclassImplementsInterface"])
+    assert "--cls.max_items" in help_str
+    with pytest.raises(ArgumentError, match="not a subclass or implementer of protocol"):
+        parser.parse_args([f"--cls.help={__name__}.NotImplementsInterface1"])
+
+    with pytest.raises(ArgumentError, match="is a protocol"):
+        parser.parse_args([f"--cls={__name__}.Interface"])
+    with pytest.raises(ArgumentError, match="does not implement protocol"):
+        parser.parse_args([f"--cls={__name__}.NotImplementsInterface1"])
+    with pytest.raises(ArgumentError, match="Does not implement protocol Interface"):
+        parser.parse_args(['--cls={"batch_size": 5}'])
+
+
+# callable protocol tests
+
+
+class CallableInterface(Protocol):
+    def __call__(self, items: List[float]) -> List[float]: ...
+
+
+class ImplementsCallableInterface:
+    def __init__(self, batch_size: int):
+        self.batch_size = batch_size
+
+    def __call__(self, items: List[float]) -> List[float]:
+        return items
+
+
+class NotImplementsCallableInterface1:
+    def __call__(self, items: str) -> List[float]:
+        return []
+
+
+class NotImplementsCallableInterface2:
+    def __call__(self, items: List[float], extra: int) -> List[float]:
+        return items
+
+
+class NotImplementsCallableInterface3:
+    def __call__(self, items: List[float]) -> None:
+        return
+
+
+@pytest.mark.parametrize(
+    "expected, value",
+    [
+        (True, ImplementsCallableInterface),
+        (False, ImplementsCallableInterface(1)),
+        (False, NotImplementsCallableInterface1),
+        (False, NotImplementsCallableInterface2),
+        (False, NotImplementsCallableInterface3),
+        (False, object),
+    ],
+)
+def test_implements_callable_protocol(expected, value):
+    assert implements_protocol(value, CallableInterface) is expected
+
+
+def test_parse_implements_callable_protocol(parser):
+    parser.add_argument("--cls", type=CallableInterface)
+    cfg = parser.parse_args([f"--cls={__name__}.ImplementsCallableInterface", "--cls.batch_size=7"])
+    assert cfg.cls.class_path == f"{__name__}.ImplementsCallableInterface"
+    assert cfg.cls.init_args == Namespace(batch_size=7)
+    init = parser.instantiate_classes(cfg)
+    assert isinstance(init.cls, ImplementsCallableInterface)
+    assert init.cls([1.0, 2.0]) == [1.0, 2.0]
+
+    assert "known subclasses:" not in get_parser_help(parser)
+    help_str = get_parse_args_stdout(parser, [f"--cls.help={__name__}.ImplementsCallableInterface"])
+    assert "--cls.batch_size" in help_str
+
+    with pytest.raises(ArgumentError, match="is a protocol"):
+        parser.parse_args([f"--cls={__name__}.CallableInterface"])
+    with pytest.raises(ArgumentError, match="does not implement protocol"):
+        parser.parse_args([f"--cls={__name__}.NotImplementsCallableInterface1"])
+    with pytest.raises(ArgumentError, match="Does not implement protocol CallableInterface"):
+        parser.parse_args(['--cls={"batch_size": 7}'])
 
 
 # parameter skip tests
@@ -1359,15 +1670,15 @@ class PrintConfig:
 
 
 def test_subclass_print_config(parser):
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
     parser.add_class_arguments(PrintConfig, "g")
 
     out = get_parse_args_stdout(parser, ["--g.a1=calendar.Calendar", "--print_config"])
-    obtained = yaml.safe_load(out)["g"]
+    obtained = json_or_yaml_load(out)["g"]
     assert obtained == {"a1": {"class_path": "calendar.Calendar", "init_args": {"firstweekday": 0}}, "a2": 7}
 
     err = get_parse_args_stderr(parser, ["--g.a1=calendar.Calendar", "--g.a1.invalid=1", "--print_config"])
-    assert 'No action for key "invalid"' in err
+    assert "Key 'invalid' is not expected" in err
 
 
 class PrintConfigRequired:
@@ -1386,7 +1697,7 @@ class PrintConfigRequiredSub(PrintConfigRequiredBase):
 
 
 def test_subclass_print_config_required_parameters_as_null(parser):
-    parser.add_argument("--config", action=ActionConfigFile)
+    parser.add_argument("--config", action="config")
     parser.add_class_arguments(PrintConfigRequired, "class")
     parser.add_subclass_arguments(PrintConfigRequiredBase, "subclass")
 
@@ -1396,22 +1707,22 @@ def test_subclass_print_config_required_parameters_as_null(parser):
         "subclass": {"class_path": f"{__name__}.PrintConfigRequiredSub", "init_args": {"arg1": None, "arg2": 1}},
     }
 
-    assert yaml.safe_load(out) == expected
+    assert json_or_yaml_load(out) == expected
 
 
 def test_subclass_multifile_save(parser, tmp_cwd):
     parser.add_subclass_arguments(Calendar, "cal")
 
     cal_cfg_path = Path("cal.yaml")
-    cal_cfg_path.write_text(yaml.dump({"class_path": "calendar.Calendar"}))
+    cal_cfg_path.write_text(json_or_yaml_dump({"class_path": "calendar.Calendar"}))
     out_main_cfg = Path("out", "config.yaml")
     out_main_cfg.parent.mkdir()
 
     cfg = parser.parse_args([f"--cal={cal_cfg_path}"])
     parser.save(cfg, out_main_cfg, multifile=True)
 
-    assert "cal: cal.yaml" == out_main_cfg.read_text().strip()
-    cal = yaml.safe_load(Path("out", "cal.yaml").read_text())
+    assert {"cal": "cal.yaml"} == json_or_yaml_load(out_main_cfg.read_text())
+    cal = json_or_yaml_load(Path("out", "cal.yaml").read_text())
     assert cal == {"class_path": "calendar.Calendar", "init_args": {"firstweekday": 0}}
 
 
@@ -1442,10 +1753,10 @@ def test_subclass_error_undefined_module(parser):
 def test_subclass_error_unexpected_init_arg(parser):
     parser.add_argument("--op", type=Calendar)
     class_path = '"class_path": "calendar.Calendar"'
-    init_args = '"init_args": {"unexpected_arg": True}'
+    init_args = '"init_args": {"unexpected_arg": true}'
     with pytest.raises(ArgumentError) as ctx:
         parser.parse_args(["--op={" + class_path + ", " + init_args + "}"])
-    ctx.match('No action for key "unexpected_arg"')
+    ctx.match("Key 'unexpected_arg' is not expected")
 
 
 def test_subclass_invalid_class_path_value(parser):
@@ -1460,7 +1771,7 @@ def test_subclass_invalid_init_args_in_yaml(parser):
         class_path: calendar.Calendar
         init_args:
     """
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     parser.add_argument("--cal", type=Calendar)
     pytest.raises(ArgumentError, lambda: parser.parse_args([f"--cfg={value}"]))
 
@@ -1500,6 +1811,23 @@ def test_subclass_help_not_subclass(parser):
     ctx.match("is not a subclass of")
 
 
+class Implicit:
+    def __init__(self, a: int = 1, b: str = ""):
+        pass
+
+
+def test_subclass_implicit_class_path(parser):
+    parser.add_argument("--implicit", type=Implicit)
+    cfg = parser.parse_args(['--implicit={"a": 2, "b": "x"}'])
+    assert cfg.implicit.class_path == f"{__name__}.Implicit"
+    assert cfg.implicit.init_args == Namespace(a=2, b="x")
+    cfg = parser.parse_args(["--implicit.a=3"])
+    assert cfg.implicit.init_args == Namespace(a=3, b="")
+    with pytest.raises(ArgumentError) as ctx:
+        parser.parse_args(['--implicit={"c": null}'])
+    ctx.match("Key 'c' is not expected")
+
+
 # error messages tests
 
 
@@ -1515,17 +1843,17 @@ def test_subclass_error_indentation_invalid_init_arg(parser):
         """
     Parser key "val":
       Does not validate against any of the Union subtypes
-      Subtypes: (<class 'int'>, <class 'dict'>, <class 'NoneType'>)
+      Subtypes: [<class 'NoneType'>, <class 'int'>, <class 'dict'>]
       Errors:
+        - Expected a <class 'NoneType'>
         - Expected a <class 'int'>
         - Expected a <class 'dict'>
-        - Expected a <class 'NoneType'>
       Given value type: <class 'str'>
       Given value: abc
     """
     ).strip()
-    expected = textwrap.indent(expected, "    ")
-    assert expected in err
+    expected = textwrap.indent(expected, "        ")
+    assert "\n".join(expected.splitlines()) in "\n".join(err.splitlines())
 
 
 class ErrorIndentation2:
@@ -1535,9 +1863,9 @@ class ErrorIndentation2:
 
 def test_subclass_error_indentation_in_union_invalid_value(parser):
     parser.add_argument("--union", type=Union[str, ErrorIndentation2])
-    parser.add_argument("--cfg", action=ActionConfigFile)
+    parser.add_argument("--cfg", action="config")
     config = {"union": [{"class_path": "ErrorIndentation2", "init_args": {"val": "x"}}]}
-    err = get_parse_args_stderr(parser, [f"--cfg={config}"])
+    err = get_parse_args_stderr(parser, [f"--cfg={json.dumps(config)}"])
     expected = textwrap.dedent(
         """
     Errors:
