@@ -15,11 +15,13 @@ import pytest
 
 from jsonargparse import (
     ArgumentParser,
-    LoggerProperty,
     Namespace,
     Path,
     capture_parser,
     class_from_function,
+)
+from jsonargparse._common import (
+    LoggerProperty,
     null_logger,
 )
 from jsonargparse._optionals import (
@@ -47,6 +49,7 @@ from jsonargparse_tests.conftest import (
     skip_if_fsspec_unavailable,
     skip_if_requests_unavailable,
     skip_if_responses_unavailable,
+    skip_if_running_as_root,
 )
 
 if responses_available:
@@ -136,6 +139,7 @@ def test_path_equality_operator(paths):
     assert Path("123", "fc") != 123
 
 
+@skip_if_running_as_root
 def test_path_file_access_mode(paths):
     Path(paths.file_rw, "frw")
     Path(paths.file_r, "fr")
@@ -150,6 +154,7 @@ def test_path_file_access_mode(paths):
     pytest.raises(TypeError, lambda: Path("file_ne", "fr"))
 
 
+@skip_if_running_as_root
 def test_path_dir_access_mode(paths):
     Path(paths.dir_rwx, "drwx")
     Path(paths.dir_rx, "drx")
@@ -166,6 +171,7 @@ def test_path_get_content(paths):
     assert "file contents" == Path(f"file://{paths.tmp_path}/{paths.file_r}", "ur").get_content()
 
 
+@skip_if_running_as_root
 def test_path_create_mode(paths):
     Path(paths.file_rw, "fcrw")
     Path(paths.tmp_path / "file_c", "fc")
@@ -533,8 +539,12 @@ def test_logger_jsonargparse_debug():
 
 
 def test_import_object_invalid():
-    pytest.raises(ValueError, lambda: import_object(True))
-    pytest.raises(ValueError, lambda: import_object("jsonargparse-tests.os"))
+    with pytest.raises(ValueError) as ctx:
+        import_object(True)
+    ctx.match("Expected a dot import path string")
+    with pytest.raises(ValueError) as ctx:
+        import_object("jsonargparse-tests.os")
+    ctx.match("Unexpected import path format")
 
 
 def test_get_import_path():
@@ -548,6 +558,36 @@ def test_get_import_path():
     assert get_import_path(MISSING) == "dataclasses.MISSING"
 
 
+class _StaticMethods:
+    @staticmethod
+    def static_method():
+        pass
+
+
+static_method = _StaticMethods.static_method
+
+
+def test_get_import_path_static_method_shorthand():
+    assert get_import_path(static_method) == f"{__name__}.static_method"
+
+
+class ParentClassmethod:
+    __module__ = "jsonargparse_tests"
+
+    @classmethod
+    def class_method(cls):
+        pass
+
+
+class ChildClassmethod(ParentClassmethod):
+    pass
+
+
+def test_get_import_path_classpath_inheritance():
+    assert get_import_path(ParentClassmethod.class_method) == "jsonargparse_tests.ParentClassmethod.class_method"
+    assert get_import_path(ChildClassmethod.class_method) == f"{__name__}.ChildClassmethod.class_method"
+
+
 def unresolvable_import():
     pass
 
